/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved.   This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 *   
 * $Id: OWLSimpleTBoxReaonerImpl.java,v 1.1.1.1 2006/01/31 09:13:26 gxie Exp $
 * 
 */
package org.eclipse.eodm.owl.reasoner.simple;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.eclipse.eodm.owl.OWLClass;
import org.eclipse.eodm.owl.OWLOntology;
import org.eclipse.eodm.owl.Property;
import org.eclipse.eodm.owl.reasoner.OWLTaxonomyReasoner;
import org.eclipse.eodm.rdfs.Namespace;
import org.eclipse.eodm.rdfs.RDFSResource;

/**
 * OWLSimpleTBoxReaonerImpl TODO Please add comments here.
 */
public class OWLSimpleTBoxReaonerImpl implements OWLTaxonomyReasoner {

    /**
     * Initialize Reasoner
     * 
     * @param ontology
     *            given ontology
     */
    public void initialize(OWLOntology ontology) {

    }

    /**
     * Get all entailed sub classes of a OWLClass by navigating the DAG
     * 
     * @param c
     *            the OWLClass to be queried
     * @return all entailed sub classes of the given OWLClass
     */
    public List getDescendantClasses(OWLClass c) {
        Vector result = new Vector();

        if (c != null) {
            long tag = getAccessTag(); // get unique access tag

            Vector queue = new Vector();
            queue.add(c);

            while (!queue.isEmpty()) {
                OWLClass node = (OWLClass) queue.remove(0);
                ResourceInfo info = infoMap.getInfo(node);
                if (info.visitTag != tag && !isInternalNode(node)) {
                    info.visitTag = tag;
                    result.add(node);
                    for (Iterator iter = node.getSubClass().iterator(); iter
                            .hasNext();) {
                        queue.add(iter.next());
                    }
                }
            }
        }

        return result;
    }

    /**
     * Get all entailed super classes of a OWLClass by navigating the DAG
     * 
     * @param c
     *            the OWLClass to be queried
     * @return all entailed super classes of the given OWLClass c
     */
    public List getAncestorClasses(OWLClass c) {
        Vector result = new Vector();

        if (c != null) {
            long tag = getAccessTag();
            Vector queue = new Vector();
            queue.add(c);

            while (!queue.isEmpty()) {
                OWLClass node = (OWLClass) queue.remove(0);
                ResourceInfo info = infoMap.getInfo(node);
                if (info.visitTag != tag && !isInternalNode(node)) {
                    info.visitTag = tag;
                    result.add(node);
                    for (Iterator iter = node.getRDFSSubClassOf().iterator(); iter
                            .hasNext();) {
                        queue.add(iter.next());
                    }
                }
            }
        }

        return result;
    }

    /**
     * Get all entailed sub properties of a Property by navigating the DAG
     * 
     * @param property
     *            the Property to be queried
     * @return all entailed sub properties of the given property
     */
    public List getDescendantProperties(Property property) {
        long tag = getAccessTag();

        Vector result = new Vector();

        if (property != null) {
            Vector queue = new Vector();
            queue.add(property);

            while (!queue.isEmpty()) {
                Property node = (Property) queue.remove(0);
                ResourceInfo info = infoMap.getInfo(node);
                if (info.visitTag != tag) {
                    info.visitTag = tag;
                    result.add(node);
                    for (Iterator iter = node.getSubProperty().iterator(); iter
                            .hasNext();) {
                        queue.add(iter.next());
                    }
                }
            }
        }

        return result;
    }

    /**
     * Get all entailed super properties of a Property by navigating the DAG
     * 
     * @param property
     *            the Property to be queried
     * @return all entailed sub properties of the given property
     */
    public List getAncestorProperties(Property property) {
        Vector result = new Vector();

        if (property != null) {
            Vector queue = new Vector();
            queue.add(property);
            long tag = getAccessTag();

            while (!queue.isEmpty()) {
                Property node = (Property) queue.remove(0);
                ResourceInfo info = infoMap.getInfo(node);
                if (info.visitTag != tag) {
                    info.visitTag = tag;
                    result.add(node);
                    for (Iterator iter = node.getRDFSSubPropertyOf().iterator(); iter
                            .hasNext();) {
                        queue.add(iter.next());
                    }
                }
            }
        }

        return result;
    }

    /**
     * Test whether two OWLClass have subClassOf relationship
     * 
     * @param c1
     *            OWLClass
     * @param c2
     *            OWLClass
     * @return true if c1 is subclass of c2
     */
    public boolean isSubClassOf(OWLClass c1, OWLClass c2) {
        if (c1 != null && c2 != null) {
            Vector queue = new Vector();
            queue.add(c2);
            long tag = getAccessTag();

            while (!queue.isEmpty()) {
                OWLClass node = (OWLClass) queue.remove(0);
                if (node == c1)
                    return true;

                ResourceInfo info = infoMap.getInfo(node);
                if (info.visitTag != tag) {
                    info.visitTag = tag;
                    for (Iterator iter = node.getSubClass().iterator(); iter
                            .hasNext();) {
                        queue.add(iter.next());
                    }
                }
            }
        }

        return false;
    }

    /**
     * Test whether two OWLClass have superClassOf relationship
     * 
     * @param c1
     *            OWLClass
     * @param c2
     *            OWLClass
     * @return true if c1 is super class of c2
     */
    public boolean isSuperClassOf(OWLClass c1, OWLClass c2) {
        return !isSubClassOf(c2, c1);
    }

    /**
     * Test whether two Property have subPropertyOf relationship
     * 
     * @param p1
     *            Property
     * @param p2
     *            Property
     * @return true if p1 is subPropertyOf of p2
     */
    public boolean isSubPropertyOf(Property p1, Property p2) {
        if (p1 != null && p2 != null) {
            Vector queue = new Vector();
            queue.add(p2);
            long tag = getAccessTag();
            while (!queue.isEmpty()) {
                Property node = (Property) queue.remove(0);
                if (node == p1)
                    return true;

                ResourceInfo info = infoMap.getInfo(node);
                if (info.visitTag != tag) {
                    info.visitTag = tag;
                    for (Iterator iter = node.getSubProperty().iterator(); iter
                            .hasNext();) {
                        queue.add(iter.next());
                    }
                }
            }
        }
        return false;
    }

    /**
     * Test whether two Property have superPropertyOf relationship
     * 
     * @param p1
     *            Property
     * @param p2
     *            Property
     * @return true if p1 is superPropertyOf of p2
     */
    public boolean isSuperPropertyOf(Property p1, Property p2) {
        return !isSubPropertyOf(p2, p1);
    }

    /*
     * Get a tag for access graph
     */
    protected long getAccessTag() {
        return lastTag++;
    }

    protected static String INTERNAL_NS = "bnode";

    protected boolean isInternalNode(RDFSResource node) {
        Namespace ns = node.getNamespace();
        return (ns == null || ns.getName().equals(INTERNAL_NS));
    }

    protected long lastTag = 1000000;

    protected ResourceInfoMap infoMap = new ResourceInfoMap();

    protected class ResourceInfoMap {
        private Map map = new HashMap();

        public ResourceInfo getInfo(Object key) {
            ResourceInfo info = (ResourceInfo) map.get(key);
            if (info == null) {
                info = new ResourceInfo();
                map.put(key, info);
            }
            return info;
        }

        public long getTag(Object key) {
            return getInfo(key).visitTag;
        }

        public void setTag(Object key, long tag) {
            getInfo(key).visitTag = tag;
        }

        public int getIndex(Object key) {
            return getInfo(key).index;
        }

        public void setIndex(Object key, int index) {
            getInfo(key).index = index;
        }
    }

    protected class ResourceInfo {
        // visiting tag for navigating the Graph
        public long visitTag = 0;

        public int range = -1;

        public int index = -1;
    }

}
