/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: RDFPropertyGraph.java,v 1.1.1.1 2006/01/31 09:13:26 gxie Exp $
 */

package org.eclipse.eodm.owl.reasoner.structural;

import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import org.eclipse.eodm.rdfs.RDFProperty;

/**
 * RDFPropertyGraph TODO Please add comments here.
 */
public class RDFPropertyGraph {
    public static final int DEFAULT_INITIAL_CAPACITY = 10;

    public RDFPropertyGraph() {
        this(DEFAULT_INITIAL_CAPACITY);
    }

    public RDFPropertyGraph(int initialCapacity) {
        size = initialCapacity;
        subPropertyMatrix = new boolean[size][size];
        properties = new String[size];
        topPropertyID = getNextID();
        bottomPropertyID = getNextID();
        subPropertyMatrix[topPropertyID][topPropertyID] = true;
        subPropertyMatrix[bottomPropertyID][topPropertyID] = true;
        subPropertyMatrix[bottomPropertyID][bottomPropertyID] = true;
    }

    public int getTopPropertyID() {
        return topPropertyID;
    }

    public int getBottomPropertyID() {
        return bottomPropertyID;
    }

    public synchronized int getCreatePropertyID(RDFProperty p) {
        String uri = p.getURI();
        Integer id = (Integer) propertyMap.get(uri);
        if (id == null) {
            id = new Integer(getNextID());
            subPropertyMatrix[id.intValue()][id.intValue()] = true;
            subPropertyMatrix[id.intValue()][topPropertyID] = true;
            subPropertyMatrix[bottomPropertyID][id.intValue()] = true;
            propertyMap.put(uri, id);
            properties[id.intValue()] = uri;
            return id.intValue();
        } else {
            return id.intValue();
        }
    }

    public synchronized int getPropertyID(RDFProperty p) {
        String uri = p.getURI();
        Integer id = (Integer) propertyMap.get(uri);
        if (id == null) {
            return -1;
        } else {
            return id.intValue();
        }
    }

    public synchronized void addSubPropertyAxiom(int p1, int p2) {
        if (p1 < 0 || p2 < 0 || p1 >= nextPropertyID || p2 >= nextPropertyID) {
            throw new ArrayIndexOutOfBoundsException();
        }
        if (subPropertyMatrix[p1][p2]) {
            return;
        }
        for (int i = 0; i < nextPropertyID; i++) {
            if (subPropertyMatrix[i][p1]) {
                for (int j = 0; j < nextPropertyID; j++) {
                    if (subPropertyMatrix[p2][j]) {
                        subPropertyMatrix[i][j] = true;
                    }
                }
            }
        }
    }

    public synchronized void addSubPropertyAxiom(RDFProperty p1, RDFProperty p2) {
        addSubPropertyAxiom(getCreatePropertyID(p1), getCreatePropertyID(p2));
    }

    /**
     * Test Whether p1 is subPropertyOf p2
     * 
     * @param p1
     * @param p2
     * @return true if p1 is subPropertyOf p2
     */
    public synchronized boolean isSubProperty(int p1, int p2) {
        if (p1 < 0 || p2 < 0 || p1 >= nextPropertyID || p2 >= nextPropertyID) {
            throw new ArrayIndexOutOfBoundsException();
        }
        return subPropertyMatrix[p1][p2];
    }

    public synchronized boolean isSubProperty(RDFProperty p1, RDFProperty p2) {
        return isSubProperty(getCreatePropertyID(p1), getCreatePropertyID(p2));
    }

    /**
     * Get Sub Properties of p
     * 
     * @param p
     *            ID of given property
     * @return URIs of all sub properties
     */
    public synchronized List getSubProperties(int p) {
        if (p < 0 || p >= nextPropertyID) {
            throw new ArrayIndexOutOfBoundsException();
        }

        Vector result = new Vector();
        for (int i = 0; i < nextPropertyID; i++) {
            if (subPropertyMatrix[i][p]
                && i != topPropertyID && i != bottomPropertyID) {
                result.add(properties[i]);
            }
        }

        return result;
    }

    /**
     * Get Sub Properties of p
     * 
     * @param p
     * @return URIs of all sub properties
     */
    public synchronized List getSubProperties(RDFProperty p) {
        return getSubProperties(getCreatePropertyID(p));
    }

    /**
     * Get Super Properties of p
     * 
     * @param p
     *            ID of given property
     * @return URIs of all super properties
     */
    public synchronized List getSuperProperties(int p) {
        if (p < 0 || p >= nextPropertyID) {
            throw new ArrayIndexOutOfBoundsException();
        }

        Vector result = new Vector();
        for (int i = 0; i < nextPropertyID; i++) {
            if (subPropertyMatrix[p][i]
                && i != topPropertyID && i != bottomPropertyID) {
                result.add(properties[i]);
            }
        }

        return result;
    }

    /**
     * Get Super Properties of p
     * 
     * @param p
     * @return URIs of all super properties
     */
    public synchronized List getSuperProperties(RDFProperty p) {
        return getSuperProperties(getCreatePropertyID(p));
    }

    private synchronized int getNextID() {
        if (nextPropertyID == size) {
            int newSize = size * 3 / 2;
            boolean[][] newMatrix = new boolean[newSize][newSize];
            String[] newPropertiesList = new String[newSize];

            for (int i = 0; i < size; i++) {
                System
                        .arraycopy(subPropertyMatrix[i], 0, newMatrix[i], 0,
                                size);
            }

            System.arraycopy(properties, 0, newPropertiesList, 0, size);

            size = newSize;
            subPropertyMatrix = newMatrix;
            properties = newPropertiesList;

        }
        return nextPropertyID++;
    }

    private HashMap propertyMap = new HashMap();

    private int nextPropertyID = 0;

    private int size;

    private boolean[][] subPropertyMatrix = null;

    private String[] properties = null;

    private int topPropertyID, bottomPropertyID;
}
