/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: OWLXMLLoader.java,v 1.1.1.1 2006/01/31 09:13:25 gxie Exp $
 */

package org.eclipse.eodm.owl.resource;

import java.io.InputStream;

import org.eclipse.eodm.owl.OWLOntology;
import org.eclipse.eodm.owl.resource.parser.OWLDocument;
import org.eclipse.eodm.owl.resource.parser.OWLParser;
import org.eclipse.eodm.owl.resource.parser.exception.OWLParserException;
import org.eclipse.eodm.owl.resource.parser.impl.OWLDocumentImpl;
import org.eclipse.eodm.owl.resource.parser.impl.OWLParserImpl;


/**
 * OWLXMLLoader
 * 
 * Load a file/stream in RDF/XML syntax to construct an ODM model.
 *  
 */
public final class OWLXMLLoader {
    private OWLXMLLoader() {
    }
    
    private static OWLParser parser=new OWLParserImpl();


    /**
     * Add a file to the loader
     * @param fileName The OWL file's name 
     * @param publicURI The file's public uri
     * @param localFile Whether it is a local file
     * @throws OWLParserException
     * 	exception when parsing or wrapping OWL file.
     */
    public static void addFile (String fileName, String publicURI, boolean localFile)
    	throws OWLParserException{
        addFile(fileName, publicURI, localFile, null);
    }
    
    /**
     * 
     * Add a file to the loader
     * @param fileName The OWL file's name 
     * @param publicURI The file's public uri
     * @param localFile Whether it is a local file
     * @param charset The charset of the file
     * @throws OWLParserException
     * 	exception when parsing or wrapping OWL file.
     */
    public static void addFile(String fileName, String publicURI, boolean localFile, String charset)
    	throws OWLParserException{
        OWLDocument document = new OWLDocumentImpl(fileName, publicURI, localFile);
        document.setCharset(charset);
      	parser.addOWLDocument(document);        
    }
    
    /**
     * Load a stream to construct an EODM OWL model
     * @param inputstream The input stream
     * @throws OWLParserException
     * 	exception when parsing or wrapping OWL file.
     */
    public static OWLOntology loadFromStream(InputStream inputstream)
		throws OWLParserException{
        return (loadFromStream(inputstream, null));
    }
    
    /**
     * Load a stream to construct an EODM OWL model
     * @param inputstream The input stream
     * @param charset The charset of the stream
     * @throws OWLParserException
     * 	exception when parsing or wrapping OWL file.
     */
    public static OWLOntology loadFromStream(InputStream inputstream, String charset)
    	throws OWLParserException{
        OWLDocument document =new OWLDocumentImpl(inputstream);
        document.setCharset(charset);
        parser.addOWLDocument(document);
        OWLOntology onto=parser.parseOWLDocument(document);
        parser.clear() ;
        return onto;
    }
    
    /**
     * Load an EODM model based on the previously added files
     * @param filename The name of the file including ontology to be formed 
     * @return EODM model
     * @throws OWLParserException
     *  exception when parsing or wrapping OWL file.
     */
    public static OWLOntology loadOntology(String filename)
    	throws OWLParserException{
        OWLOntology onto=parser.parseOWLDocument(filename);
        return onto;        
    }
    
    /**
     * Load an EODM OWL model from a file
     * @param filename The input file's name
     * @param publicURI The public URI of the file
     * @param localFile Whether the input is a local file
     * @param charset The charset of the input
     * @return EODM model
     * @throws OWLParserException
     *   exception when parsing or wrapping OWL file.
     */
    public static OWLOntology loadOntologyFromFile(String filename, String publicURI, boolean localFile, String charset)
    	throws OWLParserException{
        OWLDocument document = new OWLDocumentImpl(filename, publicURI, localFile);
        document.setCharset(charset);
      	parser.addOWLDocument(document);   
      	OWLOntology onto=parser.parseOWLDocument(document);
      	parser.clear() ;
      	return onto;
    }
    
    /**
     * Release the parser and clear all resources used
     * 
     */
    public static void clear(){
        parser.clear() ;
    }
}