/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: OWLAxiomResolver.java,v 1.1.1.1 2006/01/31 09:13:26 gxie Exp $
 */

package org.eclipse.eodm.owl.resource.parser.impl;

import java.util.List;

import org.eclipse.eodm.owl.AllValuesFromRestriction;
import org.eclipse.eodm.owl.CardinalityRestriction;
import org.eclipse.eodm.owl.ComplementClass;
import org.eclipse.eodm.owl.DatatypeSlot;
import org.eclipse.eodm.owl.EnumeratedClass;
import org.eclipse.eodm.owl.HasValueRestriction;
import org.eclipse.eodm.owl.Individual;
import org.eclipse.eodm.owl.IntersectionClass;
import org.eclipse.eodm.owl.MaxCardinalityRestriction;
import org.eclipse.eodm.owl.MinCardinalityRestriction;
import org.eclipse.eodm.owl.OWLAllDifferent;
import org.eclipse.eodm.owl.OWLClass;
import org.eclipse.eodm.owl.OWLDataRange;
import org.eclipse.eodm.owl.OWLDatatypeProperty;
import org.eclipse.eodm.owl.OWLFactory;
import org.eclipse.eodm.owl.OWLObjectProperty;
import org.eclipse.eodm.owl.OWLOntology;
import org.eclipse.eodm.owl.OWLRestriction;
import org.eclipse.eodm.owl.ObjectSlot;
import org.eclipse.eodm.owl.SomeValuesFromRestriction;
import org.eclipse.eodm.owl.UnionClass;
import org.eclipse.eodm.owl.resource.parser.OWLModelDocMapping;
import org.eclipse.eodm.owl.resource.parser.exception.OWLParserException;
import org.eclipse.eodm.owl.vocabulary.OWL;
import org.eclipse.eodm.rdf.resource.parser.element.RDFBlankNode;
import org.eclipse.eodm.rdf.resource.parser.element.RDFLiteralElement;
import org.eclipse.eodm.rdf.resource.parser.element.RDFResourceElement;
import org.eclipse.eodm.rdf.resource.parser.element.RDFSURIConstant;
import org.eclipse.eodm.rdf.resource.parser.element.RDFValue;
import org.eclipse.eodm.rdf.resource.parser.element.URIReference;
import org.eclipse.eodm.rdfs.RDFList;
import org.eclipse.eodm.rdfs.RDFProperty;
import org.eclipse.eodm.rdfs.RDFSClass;
import org.eclipse.eodm.rdfs.RDFSLiteral;
import org.eclipse.eodm.rdfs.RDFSResource;

/**
 * OWLAxiomResolver
 *  
 */
public final class OWLAxiomResolver {
    private OWLAxiomResolver() {
    }

    public static final OWLAxiomResolver _INSTANCE = new OWLAxiomResolver();

    /**
     * Ontology axioms: owl:versionInfo owl:priorVersion
     * owl:backwardCompatibleWith owl:incompatibleWith
     * 
     * Class(DataRange) axioms: owl:intersectionOf owl:unionOf owl:complementOf
     * owl:equivalentClass owl:disjointWith owl:oneOf(class-individual/data
     * range-datatype )
     * 
     * Restriction axioms: owl:onProperty owl:hasValue owl:allValuesFrom
     * owl:someValuesFrom owl:cardinality owl:maxCardinality owl:minCardinality
     * 
     * Property axioms: owl:equivalentProperty owl:inverseOf
     * 
     * Individual axioms: owl:differentFrom owl:distinctMembers owl:sameAs
     */

    public boolean resolveOWLAxiom(RDFResourceElement subject,
            URIReference predicate, RDFValue object, OWLModelDocMapping mapping) {
        if (resolveOntologyAxiom(subject, predicate, object, mapping))
            return true;
        if (resolveClassAxiom(subject, predicate, object, mapping))
            return true;
        if (resolveRestrictionAxiom(subject, predicate, object, mapping))
            return true;
        if (resolvePropertyAxiom(subject, predicate, object, mapping))
            return true;
        if (resolveIndividualAxiom(subject, predicate, object, mapping))
            return true;
        return false;
    }

    //all other triple left
    public boolean resolveOWLFact(RDFResourceElement subject,
            URIReference predicate, RDFValue object, OWLModelDocMapping mapping) {
        Individual subInv = OWLTypeIdentifier._INSTANCE.getIndividual(subject
                .getNamespace(), subject.getLocalName(), mapping);
        if (subInv == null) {
            subInv = OWLTypedElementGenerator._INSTANCE.createIndividual(
                    mapping.getModel().getNamespace(
                            ((RDFResourceElement) subject).getNamespace()),
                    ((RDFResourceElement) subject).getLocalName());
            mapping.getModel().addIndividual(subInv);
        }
        if (object instanceof RDFResourceElement) {
            OWLObjectProperty objectProperty = OWLTypeIdentifier._INSTANCE
                    .getObjectProperty(predicate.getNamespace(), predicate
                            .getLocalName(), mapping);
            if (objectProperty == null) {
                objectProperty = OWLTypedElementGenerator._INSTANCE
                        .createObjectProperty(mapping.getModel().getNamespace(
                                predicate.getNamespace()), predicate
                                .getLocalName());
                mapping.getModel().addObjectProperty(objectProperty);
            }
            Individual objInv = OWLTypeIdentifier._INSTANCE.getIndividual(
                    ((RDFResourceElement) object).getNamespace(),
                    ((RDFResourceElement) object).getLocalName(), mapping);
            if (objInv == null) {
                //throw new OWLParserException("Individual [" +
                // ((RDFResource)object).getFullURI() + " is not found.");
                objInv = OWLTypedElementGenerator._INSTANCE.createIndividual(
                        mapping.getModel().getNamespace(
                                ((RDFResourceElement) object).getNamespace()),
                        ((RDFResourceElement) object).getLocalName());
                mapping.getModel().addIndividual(objInv);
            }
            ObjectSlot slot = OWLFactory.eINSTANCE.createObjectSlot();
            slot.setIndividual(subInv);
            slot.setProperty(objectProperty);
            slot.getContent().add(objInv);
            subInv.getObjectSlot().add(slot);
        } else {
            OWLDatatypeProperty datatypeProperty = OWLTypeIdentifier._INSTANCE
                    .getDatatypeProperty(predicate.getNamespace(), predicate
                            .getLocalName(), mapping);
            if (datatypeProperty == null) {
                datatypeProperty = OWLTypedElementGenerator._INSTANCE
                        .createDatatypeProperty(mapping.getModel()
                                .getNamespace(predicate.getNamespace()),
                                predicate.getLocalName());
                mapping.getModel().addDatatypeProperty(datatypeProperty);
            }
            DatatypeSlot slot = OWLFactory.eINSTANCE.createDatatypeSlot();
            RDFSLiteral objLiteral = RDFTypeIdentifier._INSTANCE.handleLiteral(
                    (RDFLiteralElement) object, mapping);
            slot.setIndividual(subInv);
            slot.setProperty(datatypeProperty);
            slot.getContent().add(objLiteral);
            subInv.getDatatypeSlot().add(slot);
        }
        return true;
    }

    private boolean resolveOntologyAxiom(RDFResourceElement subject,
            URIReference predicate, RDFValue object, OWLModelDocMapping mapping) {
        //owl:versionInfo
        if (predicate.getFullURI().equals(OWL.VERSION_INFO_URI)) {
            RDFSLiteral info = RDFTypeIdentifier._INSTANCE.handleLiteral(
                    (RDFLiteralElement) object, mapping);
            mapping.getModel().getOntology().getOWLVersionInfo().add(info);
            return true;
        }
        //owl:priorVersion
        if (predicate.getFullURI().equals(OWL.PRIOR_VERSION_URI)) {
            URIReference objRef = (URIReference) object;
            OWLOntology prior = mapping.getModel().getOtherOnto(
                    objRef.getFullURI());
            if (prior == null) {
                prior = OWLTypedElementGenerator._INSTANCE.createOntology(
                        mapping.getModel().getNamespace(objRef.getNamespace()),
                        objRef.getLocalName());
                mapping.getModel().addOtherOnto(prior);
            }
            mapping.getModel().getOntology().getOWLPriorVersion().add(prior);
            return true;
        }
        //owl:backwardCompatibleWith
        if (predicate.getFullURI().equals(OWL.BACKWARD_COMPATIBLE_WITH_URI)) {
            URIReference objRef = (URIReference) object;
            OWLOntology backCompatible = mapping.getModel().getOtherOnto(
                    objRef.getFullURI());
            if (backCompatible == null) {
                backCompatible = OWLTypedElementGenerator._INSTANCE
                        .createOntology(mapping.getModel().getNamespace(
                                objRef.getNamespace()), objRef.getLocalName());
                mapping.getModel().addOtherOnto(backCompatible);
            }
            mapping.getModel().getOntology().getOWLBackwardCompatibleWith()
                    .add(backCompatible);
            return true;
        }
        //owl:incompatibleWith
        if (predicate.getFullURI().equals(OWL.INCOMPATIBLE_WITH_URI)) {
            URIReference objRef = (URIReference) object;
            OWLOntology incompatible = mapping.getModel().getOtherOnto(
                    objRef.getFullURI());
            if (incompatible == null) {
                incompatible = OWLTypedElementGenerator._INSTANCE
                        .createOntology(mapping.getModel().getNamespace(
                                objRef.getNamespace()), objRef.getLocalName());
                mapping.getModel().addOtherOnto(incompatible);
            }
            mapping.getModel().getOntology().getOWLIncompatibleWith().add(
                    incompatible);
            return true;
        }
        return false;
    }

    private boolean resolveClassAxiom(RDFResourceElement subject,
            URIReference predicate, RDFValue object, OWLModelDocMapping mapping) {
        //owl:intersectionOf
        if (predicate.getFullURI().equals(OWL.INTERSECTION_OF_URI)) {
            IntersectionClass subCls = (IntersectionClass) mapping.getModel()
                    .getClass(subject.getFullURI());
            constructIntersectionClass(subCls, mapping
                    .getList(((RDFBlankNode) object).getID()), mapping);
            return true;
        }
        //owl:unionOf
        if (predicate.getFullURI().equals(OWL.UNION_OF_URI)) {
            UnionClass subCls = (UnionClass) mapping.getModel().getClass(
                    subject.getFullURI());
            constructUnionClass(subCls, mapping.getList(((RDFBlankNode) object)
                    .getID()), mapping);
            return true;
        }
        //owl:complementOf
        if (predicate.getFullURI().equals(OWL.COMPLEMENT_OF_URI)) {
            ComplementClass subCls = (ComplementClass) mapping.getModel()
                    .getClass(subject.getFullURI());
            OWLClass objCls = OWLTypeIdentifier._INSTANCE.getClass(
                    ((RDFResourceElement) object).getNamespace(),
                    ((RDFResourceElement) object).getLocalName(), mapping);
            if (objCls == null)
                objCls = mapping.getModel().getRestriction(
                        ((RDFResourceElement) object).getFullURI());
            if (objCls == null)
                throw new OWLParserException("Class ["
                                             + ((RDFResourceElement) object)
                                                     .getFullURI()
                                             + "] is not found.");
            subCls.setOWLComplementOf(objCls);
            mapping.addReference(new ReferenceRecorder(subCls,
                    OWLURIConstant.COMPLEMENT_OF_URIreference, objCls));
            return true;
        }
        //owl:oneOf
        if (predicate.getFullURI().equals(OWL.ONE_OF_URI)) {
            EnumeratedClass subCls = (EnumeratedClass) OWLTypeIdentifier._INSTANCE
                    .getClass(subject.getNamespace(), subject.getLocalName(),
                            mapping);
            if (subCls != null)
                constructInvList(subCls.getOWLOneOf(), mapping
                        .getList(((RDFBlankNode) object).getID()), mapping);
            else {
                OWLDataRange dataRange = mapping.getModel().getDataRange(
                        subject.getFullURI());
                if (dataRange == null)
                    throw new OWLParserException("Class or Data Range ["
                                                 + subject.getFullURI()
                                                 + "] is not found.");
                constructLiteralList(dataRange.getOWLOneOf(), mapping
                        .getList(((RDFBlankNode) object).getID()), mapping);
            }
            return true;
        }
        //owl:equivalentClass
        if (predicate.getFullURI().equals(OWL.EQUIVALENT_CLASS_URI)) {
            OWLClass subCls = OWLTypeIdentifier._INSTANCE.getClass(subject
                    .getNamespace(), subject.getLocalName(), mapping);
            OWLClass objCls = OWLTypeIdentifier._INSTANCE.getClass(
                    ((RDFResourceElement) object).getNamespace(),
                    ((RDFResourceElement) object).getLocalName(), mapping);
            if (objCls == null) {
                objCls = OWLTypedElementGenerator._INSTANCE.createClass(mapping
                        .getModel().getNamespace(
                                ((RDFResourceElement) object).getNamespace()),
                        ((RDFResourceElement) object).getLocalName());
                mapping.getModel().addClass(objCls);
                //              throw new OWLParserException("Class [" +
                // ((RDFResource)object).getFullURI() + "] is not found.");
            }
            subCls.getOWLEquivalentClass().add(objCls);
            mapping.addReference(new ReferenceRecorder(subCls,
                    OWLURIConstant.EQUIVALENT_CLASS_URIreference, objCls));
            return true;
        }
        //owl:disjointWith
        if (predicate.getFullURI().equals(OWL.DISJOINT_WITH_URI)) {
            OWLClass subCls = OWLTypeIdentifier._INSTANCE.getClass(subject
                    .getNamespace(), subject.getLocalName(), mapping);
            OWLClass objCls = OWLTypeIdentifier._INSTANCE.getClass(
                    ((RDFResourceElement) object).getNamespace(),
                    ((RDFResourceElement) object).getLocalName(), mapping);
            if (objCls == null) {
                objCls = OWLTypedElementGenerator._INSTANCE.createClass(mapping
                        .getModel().getNamespace(
                                ((RDFResourceElement) object).getNamespace()),
                        ((RDFResourceElement) object).getLocalName());
                mapping.getModel().addClass(objCls);
                //                throw new OWLParserException("Class [" +
                // ((RDFResource)object).getFullURI() + "] is not found.");
            }
            subCls.getOWLDisjointWith().add(objCls);
            mapping.addReference(new ReferenceRecorder(subCls,
                    OWLURIConstant.DISJOINT_WITH_URIreference, objCls));
            return true;
        }
        return false;
    }

    private boolean resolveRestrictionAxiom(RDFResourceElement subject,
            URIReference predicate, RDFValue object, OWLModelDocMapping mapping) {
        //owl:onProperty
        if (predicate.getFullURI().equals(OWL.ON_PROPERTY_URI)) {
            OWLRestriction restriction = mapping.getModel().getRestriction(
                    subject.getFullURI());
            URIReference objRef = (URIReference) object;
            RDFProperty property = OWLTypeIdentifier._INSTANCE
                    .getObjectProperty(objRef.getNamespace(), objRef
                            .getLocalName(), mapping);
            if (property == null)
                property = OWLTypeIdentifier._INSTANCE.getDatatypeProperty(
                        objRef.getNamespace(), objRef.getLocalName(), mapping);
            if (property == null)
                throw new OWLParserException("Property ["
                                             + objRef.getFullURI()
                                             + "] is not found.");
            restriction.setOWLOnProperty(property);
            return true;
        }
        //owl:allValuesFrom
        if (predicate.getFullURI().equals(OWL.ALL_VALUES_FROM_URI)) {
            AllValuesFromRestriction restriction = (AllValuesFromRestriction) mapping
                    .getModel().getRestriction(subject.getFullURI());
            RDFResourceElement objRes = (RDFResourceElement) object;
            RDFSClass cls = OWLTypeIdentifier._INSTANCE.getClass(objRes
                    .getNamespace(), objRes.getLocalName(), mapping);
            if (cls == null)
                cls = mapping.getModel().getRestriction(objRes.getFullURI());
            if (cls == null)
                cls = mapping.getModel().getDataRange(objRes.getFullURI());
            if (cls == null)
                cls = OWLTypeIdentifier._INSTANCE.getDatatype(objRes
                        .getNamespace(), objRes.getLocalName(), mapping);
            if (cls == null)
                throw new OWLParserException("Class ["
                                             + objRes.getFullURI()
                                             + "] is not found.");
            restriction.setOWLAllValuesFrom(cls);
            mapping.addReference(new ReferenceRecorder(restriction,
                    OWLURIConstant.ALL_VALUES_FROM_URIreference, cls));
            return true;
        }
        //owl:someValuesFrom
        if (predicate.getFullURI().equals(OWL.SOME_VALUES_FROM_URI)) {
            SomeValuesFromRestriction restriction = (SomeValuesFromRestriction) mapping
                    .getModel().getRestriction(subject.getFullURI());
            RDFResourceElement objRes = (RDFResourceElement) object;
            RDFSClass cls = OWLTypeIdentifier._INSTANCE.getClass(objRes
                    .getNamespace(), objRes.getLocalName(), mapping);
            if (cls == null)
                cls = mapping.getModel().getRestriction(objRes.getFullURI());
            if (cls == null)
                cls = mapping.getModel().getDataRange(objRes.getFullURI());
            if (cls == null)
                cls = OWLTypeIdentifier._INSTANCE.getDatatype(objRes
                        .getNamespace(), objRes.getLocalName(), mapping);
            if (cls == null)
                throw new OWLParserException("Class ["
                                             + objRes.getFullURI()
                                             + "] is not found.");
            restriction.setOWLSomeValuesFrom(cls);
            mapping.addReference(new ReferenceRecorder(restriction,
                    OWLURIConstant.SOME_VALUES_FROM_URIreference, cls));
            return true;
        }
        //owl:hasValue
        if (predicate.getFullURI().equals(OWL.HAS_VALUE_URI)) {
            HasValueRestriction restriction = (HasValueRestriction) mapping
                    .getModel().getRestriction(subject.getFullURI());
            RDFSResource value = null;
            if (object instanceof RDFLiteralElement)
                value = RDFTypeIdentifier._INSTANCE.handleLiteral(
                        (RDFLiteralElement) object, mapping);
            else {
                RDFResourceElement objRes = (RDFResourceElement) object;
                value = OWLTypeIdentifier._INSTANCE.getIndividual(objRes
                        .getNamespace(), objRes.getLocalName(), mapping);
                if (value == null)
                    throw new OWLParserException("Value ["
                                                 + objRes.getFullURI()
                                                 + "] is not found.");
            }
            restriction.setOWLHasValue(value);
            return true;
        }
        //owl:cardinality
        if (predicate.getFullURI().equals(OWL.CARDINALITY_URI)) {
            CardinalityRestriction restriction = (CardinalityRestriction) mapping
                    .getModel().getRestriction(subject.getFullURI());
            restriction.setOWLCardinality(RDFTypeIdentifier._INSTANCE
                    .handleLiteral((RDFLiteralElement) object, mapping));
            return true;
        }
        //owl:maxCardinality
        if (predicate.getFullURI().equals(OWL.MAX_CARDINALITY_URI)) {
            MaxCardinalityRestriction restriction = (MaxCardinalityRestriction) mapping
                    .getModel().getRestriction(subject.getFullURI());
            restriction.setOWLMaxCardinality(RDFTypeIdentifier._INSTANCE
                    .handleLiteral((RDFLiteralElement) object, mapping));
            return true;
        }
        //owl:minCardinality
        if (predicate.getFullURI().equals(OWL.MIN_CARDINALITY_URI)) {
            MinCardinalityRestriction restriction = (MinCardinalityRestriction) mapping
                    .getModel().getRestriction(subject.getFullURI());
            restriction.setOWLMinCardinality(RDFTypeIdentifier._INSTANCE
                    .handleLiteral((RDFLiteralElement) object, mapping));
            return true;
        }
        return false;
    }

    private boolean resolvePropertyAxiom(RDFResourceElement subject,
            URIReference predicate, RDFValue object, OWLModelDocMapping mapping) {
        //owl:equivalentProperty
        if (predicate.getFullURI().equals(OWL.EQUIVALENT_PROPERTY_URI)) {
            OWLObjectProperty subObjPro = OWLTypeIdentifier._INSTANCE
                    .getObjectProperty(subject.getNamespace(), subject
                            .getLocalName(), mapping);
            if (subObjPro != null) {
                OWLObjectProperty objObjPro = OWLTypeIdentifier._INSTANCE
                        .getObjectProperty(((RDFResourceElement) object)
                                .getNamespace(), ((RDFResourceElement) object)
                                .getLocalName(), mapping);
                if (objObjPro == null)
                    throw new OWLParserException(
                            "Object Property ["
                                                                                                                + ((RDFResourceElement) object)
                                                                                                                        .getFullURI()
                                                                                                                + "] is not found.");
                subObjPro.getOWLEquivalentProperty().add(objObjPro);
            } else {
                OWLDatatypeProperty subDataPro = OWLTypeIdentifier._INSTANCE
                        .getDatatypeProperty(subject.getNamespace(), subject
                                .getLocalName(), mapping);
                if (subDataPro == null)
                    throw new OWLParserException("Property ["
                                                 + subject.getFullURI()
                                                 + "] is not found.");
                OWLDatatypeProperty objDataPro = OWLTypeIdentifier._INSTANCE
                        .getDatatypeProperty(((RDFResourceElement) object)
                                .getNamespace(), ((RDFResourceElement) object)
                                .getLocalName(), mapping);
                if (objDataPro == null)
                    throw new OWLParserException(
                            "Property ["
                                                                                                                + ((RDFResourceElement) object)
                                                                                                                        .getFullURI()
                                                                                                                + "] is not found.");
                subDataPro.getOWLEquivalentProperty().add(objDataPro);
            }
            return true;
        }
        //owl:inverseOf
        if (predicate.getFullURI().equals(OWL.INVERSE_OF_URI)) {
            OWLObjectProperty subPro = OWLTypeIdentifier._INSTANCE
                    .getObjectProperty(subject.getNamespace(), subject
                            .getLocalName(), mapping);
            if (subPro == null)
                throw new OWLParserException("Property ["
                                             + subPro.getURI()
                                             + "] is not found.");
            OWLObjectProperty objPro = OWLTypeIdentifier._INSTANCE
                    .getObjectProperty(((RDFResourceElement) object)
                            .getNamespace(), ((RDFResourceElement) object)
                            .getLocalName(), mapping);
            if (objPro == null)
                throw new OWLParserException("Property ["
                                             + objPro.getURI()
                                             + "] is not found.");
            subPro.setOWLInverseOf(objPro);
            return true;
        }
        return false;
    }

    private boolean resolveIndividualAxiom(RDFResourceElement subject,
            URIReference predicate, RDFValue object, OWLModelDocMapping mapping) {
        //owl:differentFrom
        if (predicate.getFullURI().equals(OWL.DIFFERENT_FROM_URI)) {
            Individual subInv = OWLTypeIdentifier._INSTANCE.getIndividual(
                    subject.getNamespace(), subject.getLocalName(), mapping);
            Individual objInv = OWLTypeIdentifier._INSTANCE.getIndividual(
                    ((RDFResourceElement) object).getNamespace(),
                    ((RDFResourceElement) object).getLocalName(), mapping);
            if (subInv == null)
                throw new OWLParserException("Individual ["
                                             + subject.getFullURI()
                                             + "] is not found.");
            if (objInv == null)
                throw new OWLParserException("Individual ["
                                             + ((RDFResourceElement) object)
                                                     .getFullURI()
                                             + "] is not found.");
            subInv.getOWLDifferentFrom().add(objInv);
            return true;
        }
        //owl:distinctMembers
        if (predicate.getFullURI().equals(OWL.DISTINCT_MEMBERS_URI)) {
            OWLAllDifferent allDiff = mapping.getModel().getAllDifferent(
                    subject.getLocalName());
            constructInvList(allDiff.getOWLDistinctMembers(), mapping
                    .getList(((RDFBlankNode) object).getID()), mapping);
            for (int i = 0; i < allDiff.getOWLDistinctMembers().size(); i++) {
                Individual inv1 = (Individual) allDiff.getOWLDistinctMembers()
                        .get(i);
                for (int j = i + 1; j < allDiff.getOWLDistinctMembers().size(); j++) {
                    Individual inv2 = (Individual) allDiff
                            .getOWLDistinctMembers().get(j);
                    if (!inv1.getOWLDifferentFrom().contains(inv2))
                        inv1.getOWLDifferentFrom().add(inv2);
                    if (!inv2.getOWLDifferentFrom().contains(inv1))
                        inv2.getOWLDifferentFrom().add(inv1);
                }
            }
            //allDiff.getOWLDistinctMembers().clear();
            mapping.getModel().getOntology().getContains().add(allDiff);
            return true;
        }
        //owl:sameAs
        if (predicate.getFullURI().equals(OWL.SAME_AS_URI)) {
            Individual subInv = OWLTypeIdentifier._INSTANCE.getIndividual(
                    subject.getNamespace(), subject.getLocalName(), mapping);
            Individual objInv = OWLTypeIdentifier._INSTANCE.getIndividual(
                    ((RDFResourceElement) object).getNamespace(),
                    ((RDFResourceElement) object).getLocalName(), mapping);
            if (subInv == null)
                throw new OWLParserException("Individual ["
                                             + subject.getFullURI()
                                             + "] is not found.");
            if (objInv == null)
                throw new OWLParserException("Individual ["
                                             + ((RDFResourceElement) object)
                                                     .getFullURI()
                                             + "] is not found.");
            subInv.getOWLSameAs().add(objInv);
            return true;
        }
        return false;
    }

    private void constructIntersectionClass(IntersectionClass interClass,
            RDFList list, OWLModelDocMapping mapping) {
        RDFList tmp = list;
        while (tmp != null && !tmp.getLocalName().equals(RDFSURIConstant.NIL)) {
            RDFSResource first = tmp.getRDFFirst();
            OWLClass cls = OWLTypeIdentifier._INSTANCE.getClass(first
                    .getNamespace().getURI(), first.getLocalName(), mapping);
            if (cls == null)
                cls = mapping.getModel().getRestriction(first.getURI());
            if (cls == null)
                throw new OWLParserException("Class["
                                             + first.getURI()
                                             + " is not found.");
            interClass.getOWLIntersectionOf().add(cls);
            mapping.addReference(new ReferenceRecorder(interClass,
                    OWLURIConstant.INTERSECTION_OF_URIreference, cls));
            tmp = tmp.getRDFRest();
        }
    }

    private void constructUnionClass(UnionClass unionClass, RDFList list,
            OWLModelDocMapping mapping) {
        RDFList tmp = list;
        while (tmp != null && !tmp.getLocalName().equals(RDFSURIConstant.NIL)) {
            RDFSResource first = tmp.getRDFFirst();
            OWLClass cls = OWLTypeIdentifier._INSTANCE.getClass(first
                    .getNamespace().getURI(), first.getLocalName(), mapping);
            if (cls == null)
                cls = mapping.getModel().getRestriction(first.getURI());
            if (cls == null)
                throw new OWLParserException("Class["
                                             + first.getURI()
                                             + " is not found.");
            unionClass.getOWLUnionOf().add(cls);
            mapping.addReference(new ReferenceRecorder(unionClass,
                    OWLURIConstant.UNION_OF_URIreference, cls));
            tmp = tmp.getRDFRest();
        }
    }

    private void constructInvList(List contents, RDFList list,
            OWLModelDocMapping mapping) {
        RDFList tmp = list;
        while (tmp != null && !tmp.getLocalName().equals(RDFSURIConstant.NIL)) {
            RDFSResource first = tmp.getRDFFirst();
            Individual inv = OWLTypeIdentifier._INSTANCE.getIndividual(first
                    .getNamespace().getURI(), first.getLocalName(), mapping);
            if (inv == null)
                throw new OWLParserException("Individual["
                                             + first.getURI()
                                             + " is not found.");
            contents.add(inv);
            tmp = tmp.getRDFRest();
        }
    }

    private void constructLiteralList(List contents, RDFList list,
            OWLModelDocMapping mapping) {
        RDFList tmp = list;
        while (tmp != null && !tmp.getLocalName().equals(RDFSURIConstant.NIL)) {
            RDFSResource first = tmp.getRDFFirst();
            contents.add(first);
            tmp = tmp.getRDFRest();
        }
    }
}