/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: OWLTypeIdentifier.java,v 1.1.1.1 2006/01/31 09:13:26 gxie Exp $
 */

package org.eclipse.eodm.owl.resource.parser.impl;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.eodm.owl.Individual;
import org.eclipse.eodm.owl.OWLAllDifferent;
import org.eclipse.eodm.owl.OWLAnnotationProperty;
import org.eclipse.eodm.owl.OWLClass;
import org.eclipse.eodm.owl.OWLDataRange;
import org.eclipse.eodm.owl.OWLDatatypeProperty;
import org.eclipse.eodm.owl.OWLObjectProperty;
import org.eclipse.eodm.owl.OWLOntology;
import org.eclipse.eodm.owl.OWLOntologyProperty;
import org.eclipse.eodm.owl.OWLRestriction;
import org.eclipse.eodm.owl.resource.parser.OWLDocument;
import org.eclipse.eodm.owl.resource.parser.OWLModelDocMapping;
import org.eclipse.eodm.owl.resource.parser.OWLModelDocRepository;
import org.eclipse.eodm.owl.resource.parser.OWLOntoModel;
import org.eclipse.eodm.owl.resource.parser.exception.OWLParserException;
import org.eclipse.eodm.owl.vocabulary.OWL;
import org.eclipse.eodm.rdf.resource.parser.element.RDFResourceElement;
import org.eclipse.eodm.rdf.resource.parser.element.RDFSURIConstant;
import org.eclipse.eodm.rdf.resource.parser.element.RDFValue;
import org.eclipse.eodm.rdf.resource.parser.element.URIReference;
import org.eclipse.eodm.rdfs.RDFSDatatype;
import org.eclipse.eodm.rdfs.vocabulary.RDF;

/**
 * OWLTypeIdentifier
 *  
 */
public final class OWLTypeIdentifier {
    private OWLTypeIdentifier() {
    }

    //Store URIs of ontology from the DB
    private List URIList = null;

    public void setURIList(List uri) {
        URIList = uri;
    }

    /** Single instance of OWLTypeIdentifier */
    public static final OWLTypeIdentifier _INSTANCE = new OWLTypeIdentifier();

    /**
     * Identify all the typed elements(including named objects and blank nodes).
     * e.g, Ontologies, Classes, Properties(Object Properties & Datatype
     * Properties), Restrictions, Individuals, DataRanges, AllDifferents.
     * <p>
     * Handle triples like <subject rdf:type object> with predicate of
     * "rdf:type".
     * 
     * @param subject
     *            the subject of the triple
     * @param predicate
     *            the predicate of the triple
     * @param object
     *            the object of the triple
     * @param tripleAnalyser
     *            the triple analyser
     */
    public boolean identifyOWLType1(RDFResourceElement subject,
            URIReference predicate, RDFValue object, OWLModelDocMapping mapping) {
        if (!predicate.getFullURI().equals(RDF.P_TYPE_STR))
            return false;

        /* Object must be a URIreference */
        URIReference objRef = (URIReference) object;

        /* Create different element according to the type */
        int type = OWLTypeConstants.getOWLTypeByURI(objRef.getFullURI());
        switch (type) {
        case OWLTypeConstants.OWL_TYPE_ONTOLOGY:
            handleOntology(subject.getNamespace(), subject.getLocalName(),
                    mapping);
            return true;
        case OWLTypeConstants.OWL_TYPE_CLASS:
            handleClass(subject.getNamespace(), subject.getLocalName(), mapping);
            return true;
        case OWLTypeConstants.OWL_TYPE_RESTRICTION:
            handleRestriction(subject.getNamespace(), subject.getLocalName(),
                    mapping);
            return true;
        case OWLTypeConstants.OWL_TYPE_OBJECT_PROPERTY:
            handleObjProperty(subject.getNamespace(), subject.getLocalName(),
                    mapping);
            return true;
        case OWLTypeConstants.OWL_TYPE_DATATYPE_PROPERTY:
            handleDatatypeProperty(subject.getNamespace(), subject
                    .getLocalName(), mapping);
            return true;
        case OWLTypeConstants.OWL_TYPE_DATARANGE:
            handleDataRange(subject.getNamespace(), subject.getLocalName(),
                    mapping);
            return true;
        case OWLTypeConstants.OWL_TYPE_ANNOTATION_PROPERTY:
            handleAnnoProperty(subject.getNamespace(), subject.getLocalName(),
                    mapping);
            return true;
        case OWLTypeConstants.OWL_TYPE_ONTOLOGY_PROPERTY:
            handleOntoProperty(subject.getNamespace(), subject.getLocalName(),
                    mapping);
            return true;
        case OWLTypeConstants.OWL_TYPE_ALL_DIFFERENT:
            handleAllDifferent(subject.getNamespace(), subject.getLocalName(),
                    mapping);
            return true;
        }

        return false;
    }

    //owl:import
    public boolean identifyOWLImport(RDFResourceElement subject,
            URIReference predicate, RDFValue object, OWLModelDocMapping mapping) {
        if (!predicate.getFullURI().equals(OWL.IMPORTS_URI))
            return false;
        if (URIList != null) {
            for (int i = 0; i < URIList.size(); i++) {
                if (((URIReference) object).getFullURI().equals(URIList.get(i)))
                    return true;
            }
        }
        mapping.addImpURI(((URIReference) object).getFullURI());
        return true;
    }

    public boolean identifyOWLType2(RDFResourceElement subject,
            URIReference predicate, RDFValue object,
            OWLModelDocRepository repository, OWLModelDocMapping mapping) {
        if (predicate.getFullURI().equals(RDF.P_TYPE_STR)) {
            URIReference objRef = (URIReference) object;
            int type = OWLTypeConstants.getOWLTypeByURI(objRef.getFullURI());
            /**
             * 1. Suppose all deprecated classes must have been defined as typed
             * classes with URIreference, deprecated properties, functional
             * properties, inverse functional properties, symmetric properties,
             * transitive properties must have been defined as properties with
             * URI. 2. Individuals' type should be those classes which are
             * defined explicitly with URIreferences.
             */
            switch (type) {
            case OWLTypeConstants.OWL_TYPE_DEPRECATED_CLASS:
                handleDeprecatedClass(subject.getNamespace(), subject
                        .getLocalName(), mapping);
                return true;
            case OWLTypeConstants.OWL_TYPE_DEPRECATED_PROPERTY:
                handleDeprecatedProperty(subject.getNamespace(), subject
                        .getLocalName(), mapping);
                return true;
            case OWLTypeConstants.OWL_TYPE_FUNCTIONAL_PROPERTY:
                handleFunctionalProperty(subject.getNamespace(), subject
                        .getLocalName(), mapping);
                return true;
            case OWLTypeConstants.OWL_TYPE_INVERSE_FUNCTIONAL_PROPERTY:
                handleInverseFunctionalProperty(subject.getNamespace(), subject
                        .getLocalName(), mapping);
                return true;
            case OWLTypeConstants.OWL_TYPE_SYMMETRIC_PROPERTY:
                handleSymmetricProperty(subject.getNamespace(), subject
                        .getLocalName(), mapping);
                return true;
            case OWLTypeConstants.OWL_TYPE_TRANSITIVE_PROPERTY:
                handleTransitiveProperty(subject.getNamespace(), subject
                        .getLocalName(), mapping);
                return true;
            case OWLTypeConstants.OWL_TYPE_THING:
            case OWLTypeConstants.OWL_TYPE_UNKNOWN:
                handleIndividual(subject, objRef, repository, mapping);
                return true;
            }
        }

        if (predicate.getFullURI().equals(OWL.ALL_VALUES_FROM_URI)
            || predicate.getFullURI().equals(OWL.SOME_VALUES_FROM_URI)
            || predicate.getFullURI().equals(OWL.HAS_VALUE_URI)) {
            handleValueRestriction(subject, predicate.getFullURI(), mapping);
            return false;
        }
        if (predicate.getFullURI().equals(OWL.CARDINALITY_URI)
            || predicate.getFullURI().equals(OWL.MAX_CARDINALITY_URI)
            || predicate.getFullURI().equals(OWL.MIN_CARDINALITY_URI)) {
            handleCardinalityRestriction(subject, predicate.getFullURI(),
                    mapping);
            return false;
        }
        if (predicate.getFullURI().equals(OWL.INTERSECTION_OF_URI)
            || predicate.getFullURI().equals(OWL.UNION_OF_URI)
            || predicate.getFullURI().equals(OWL.COMPLEMENT_OF_URI)) {
            handleClassExpression(subject, predicate.getFullURI(), repository,
                    mapping);
            return false;
        }
        if (predicate.getFullURI().equals(OWL.ONE_OF_URI)) {
            handleOneOf(subject, repository, mapping);
            return false;
        }

        return false;
    }

    private void handleOntology(String namespace, String localName,
            OWLModelDocMapping mapping) {
        OWLOntology ontology = mapping.getModel().getOntology();
        if (ontology.getNamespace() == null) {
            ontology.setNamespace(mapping.getModel().getNamespace(namespace));
            ontology.setLocalName(localName);
        } else {
            //Prior version
            OWLOntology onto = OWLTypedElementGenerator._INSTANCE
                    .createOntology(mapping.getModel().getNamespace(namespace),
                            localName);
            mapping.getModel().addOtherOnto(onto);
        }

        //Set Public URI for documents
        List docList = mapping.getDocs();
        Iterator iter = docList.iterator();
        OWLDocument doc = null;
        while (iter.hasNext()) {
            doc = (OWLDocument) (iter.next());
            doc.setPublicURI(ontology.getURI());
        }
    }

    private void handleClass(String namespace, String localName,
            OWLModelDocMapping mapping) {
        OWLClass cls = OWLTypedElementGenerator._INSTANCE.createClass(mapping
                .getModel().getNamespace(namespace), localName);
        mapping.getModel().addClass(cls);
    }

    private void handleDeprecatedClass(String namespace, String localName,
            OWLModelDocMapping mapping) {
        //get the class in the current model and all the imported models
        OWLClass cls = getClass(namespace, localName, mapping);
        if (cls != null) {
            cls.setDeprecated(Boolean.TRUE);
            return;
        }
        throw new OWLParserException("Class ["
                                     + namespace + localName
                                     + "] is not found.");
    }

    private void handleRestriction(String namespace, String localName,
            OWLModelDocMapping mapping) {
        mapping.addRestriction(localName);
    }

    private void handleObjProperty(String namespace, String localName,
            OWLModelDocMapping mapping) {
        OWLObjectProperty objProperty = OWLTypedElementGenerator._INSTANCE
                .createObjectProperty(mapping.getModel()
                        .getNamespace(namespace), localName);
        mapping.getModel().addObjectProperty(objProperty);
    }

    private void handleDatatypeProperty(String namespace, String localName,
            OWLModelDocMapping mapping) {
        OWLDatatypeProperty dataProperty = OWLTypedElementGenerator._INSTANCE
                .createDatatypeProperty(mapping.getModel().getNamespace(
                        namespace), localName);
        mapping.getModel().addDatatypeProperty(dataProperty);
    }

    private void handleAnnoProperty(String namespace, String localName,
            OWLModelDocMapping mapping) {
        OWLAnnotationProperty annoProperty = OWLTypedElementGenerator._INSTANCE
                .createAnnotationProperty(mapping.getModel().getNamespace(
                        namespace), localName);
        mapping.getModel().addAnnoProperty(annoProperty);
    }

    private void handleOntoProperty(String namespace, String localName,
            OWLModelDocMapping mapping) {
        OWLOntologyProperty ontoProperty = OWLTypedElementGenerator._INSTANCE
                .createOntologyProperty(mapping.getModel().getNamespace(
                        namespace), localName);
        mapping.getModel().addOntoProperty(ontoProperty);
    }

    private void handleDeprecatedProperty(String namespace, String localName,
            OWLModelDocMapping mapping) {
        //get the object property in the current model and all imported models
        OWLObjectProperty objProperty = getObjectProperty(namespace, localName,
                mapping);
        if (objProperty != null) {
            objProperty.setDeprecated(Boolean.TRUE);
            return;
        }
        //get the datatype property in the current model and all imported
        // models
        OWLDatatypeProperty dataProperty = getDatatypeProperty(namespace,
                localName, mapping);
        if (dataProperty != null) {
            dataProperty.setDeprecated(Boolean.TRUE);
            return;
        }
        throw new OWLParserException("Property ["
                                     + namespace + localName
                                     + "] is not found.");
    }

    private void handleFunctionalProperty(String namespace, String localName,
            OWLModelDocMapping mapping) {
        //get the object property in the current model and all imported models
        OWLObjectProperty objProperty = getObjectProperty(namespace, localName,
                mapping);
        if (objProperty != null) {
            objProperty.setFunctional(Boolean.TRUE);
            return;
        }
        //get the datatype property in the current model and all imported
        // models
        OWLDatatypeProperty dataProperty = getDatatypeProperty(namespace,
                localName, mapping);
        if (dataProperty != null) {
            dataProperty.setFunctional(Boolean.TRUE);
            return;
        }
        throw new OWLParserException("Property ["
                                     + namespace + localName
                                     + "] is not found.");
    }

    private void handleInverseFunctionalProperty(String namespace,
            String localName, OWLModelDocMapping mapping) {
        //get the object property in the current model and all imported models
        OWLObjectProperty objProperty = getObjectProperty(namespace, localName,
                mapping);
        if (objProperty != null) {
            objProperty.setInverseFunctional(Boolean.TRUE);
            return;
        }
        //add an object property (notthrow exception)
        objProperty = OWLTypedElementGenerator._INSTANCE.createObjectProperty(
                mapping.getModel().getNamespace(namespace), localName);
        objProperty.setInverseFunctional(Boolean.TRUE);
        mapping.getModel().addObjectProperty(objProperty);
    }

    private void handleSymmetricProperty(String namespace, String localName,
            OWLModelDocMapping mapping) {
        //get the object property in the current model and all imported models
        OWLObjectProperty objProperty = getObjectProperty(namespace, localName,
                mapping);
        if (objProperty != null) {
            objProperty.setSymmetric(Boolean.TRUE);
            return;
        }
        //add an object property (not throw exception)
        objProperty = OWLTypedElementGenerator._INSTANCE.createObjectProperty(
                mapping.getModel().getNamespace(namespace), localName);
        objProperty.setSymmetric(Boolean.TRUE);
        mapping.getModel().addObjectProperty(objProperty);
    }

    private void handleTransitiveProperty(String namespace, String localName,
            OWLModelDocMapping mapping) {
        //get the object property in the current model and all imported models
        OWLObjectProperty objProperty = getObjectProperty(namespace, localName,
                mapping);
        if (objProperty != null) {
            objProperty.setTransitive(Boolean.TRUE);
            return;
        }
        //add an object property
        objProperty = OWLTypedElementGenerator._INSTANCE.createObjectProperty(
                mapping.getModel().getNamespace(namespace), localName);
        objProperty.setTransitive(Boolean.TRUE);
        mapping.getModel().addObjectProperty(objProperty);
    }

    private void handleIndividual(RDFResourceElement subject,
            URIReference object, OWLModelDocRepository repository,
            OWLModelDocMapping mapping) {
        //get the individual in the current model and all imported models
        Individual individual = getIndividual(subject.getNamespace(), subject
                .getLocalName(), mapping);
        if (individual == null) {
            individual = OWLTypedElementGenerator._INSTANCE.createIndividual(
                    mapping.getModel().getNamespace(subject.getNamespace()),
                    subject.getLocalName());
            mapping.getModel().addIndividual(individual);
        }
        OWLClass type = getClass(object.getNamespace(), object.getLocalName(),
                mapping);
        if (type == null)
            throw new OWLParserException("Class ["
                                         + object.getFullURI()
                                         + "] is not defined.");
        //rdf:type <RDFSClass>
        List types = individual.getRDFType();
        for (int i = 0; i < types.size(); i++)
            if (((OWLClass) types.get(i)).getURI().equals(object.getFullURI()))
                return;
        individual.getRDFType().add(type);
        repository.addReference(new ReferenceRecorder(individual,
                RDFSURIConstant.P_TYPE_URI, type));
    }

    private void handleDataRange(String namespace, String localName,
            OWLModelDocMapping mapping) {
        OWLDataRange dataRange = OWLTypedElementGenerator._INSTANCE
                .createDataRange(mapping.getModel().getNamespace(namespace),
                        localName);
        mapping.getModel().addDataRange(dataRange);
    }

    private void handleAllDifferent(String namespace, String localName,
            OWLModelDocMapping mapping) {
        OWLAllDifferent allDiff = OWLTypedElementGenerator._INSTANCE
                .createAllDifferent();
        mapping.getModel().addAllDifferent(localName, allDiff);
    }

    //value restriction
    private void handleValueRestriction(RDFResourceElement subject,
            String predicate, OWLModelDocMapping mapping) {
        boolean contains = mapping.containsRestriction(subject.getLocalName());
        if (!contains)
            throw new OWLParserException("Restriction ["
                                         + subject.getLocalName()
                                         + "] is not defined.");
        OWLRestriction restriction = null;
        if (predicate.equals(OWL.ALL_VALUES_FROM_URI))
            restriction = OWLTypedElementGenerator._INSTANCE
                    .createAllValuesFromRestriction(mapping.getModel()
                            .getNamespace(subject.getNamespace()), subject
                            .getLocalName());
        if (predicate.equals(OWL.SOME_VALUES_FROM_URI))
            restriction = OWLTypedElementGenerator._INSTANCE
                    .createSomeValuesFromRestriction(mapping.getModel()
                            .getNamespace(subject.getNamespace()), subject
                            .getLocalName());
        if (predicate.equals(OWL.HAS_VALUE_URI))
            restriction = OWLTypedElementGenerator._INSTANCE
                    .createHasValueRestriction(mapping.getModel().getNamespace(
                            subject.getNamespace()), subject.getLocalName());
        mapping.getModel().addRestriction(restriction);
    }

    //cardinality restriction
    private void handleCardinalityRestriction(RDFResourceElement subject,
            String predicate, OWLModelDocMapping mapping) {
        boolean contains = mapping.containsRestriction(subject.getLocalName());
        if (!contains)
            throw new OWLParserException("Restriction ["
                                         + subject.getLocalName()
                                         + "] is not defined.");
        OWLRestriction restriction = null;
        if (predicate.equals(OWL.CARDINALITY_URI))
            restriction = OWLTypedElementGenerator._INSTANCE
                    .createCardinalityRestriction(mapping.getModel()
                            .getNamespace(subject.getNamespace()), subject
                            .getLocalName());
        if (predicate.equals(OWL.MAX_CARDINALITY_URI))
            restriction = OWLTypedElementGenerator._INSTANCE
                    .createMaxCardinalityRestriction(mapping.getModel()
                            .getNamespace(subject.getNamespace()), subject
                            .getLocalName());
        if (predicate.equals(OWL.MIN_CARDINALITY_URI))
            restriction = OWLTypedElementGenerator._INSTANCE
                    .createMinCardinalityRestriction(mapping.getModel()
                            .getNamespace(subject.getNamespace()), subject
                            .getLocalName());
        mapping.getModel().addRestriction(restriction);
    }

    private void handleClassExpression(RDFResourceElement subject,
            String predicate, OWLModelDocRepository repository,
            OWLModelDocMapping mapping) {
        OWLClass cls = mapping.getModel().getClass(subject.getFullURI());
        if (cls == null)
            throw new OWLParserException("Class ["
                                         + subject.getFullURI()
                                         + "] is not defined.");
        mapping.getModel().getClasses().remove(subject.getFullURI());
        mapping.getModel().getOntology().getContains().remove(cls);
        OWLClass expCls = null;
        if (predicate.equals(OWL.INTERSECTION_OF_URI))
            expCls = OWLTypedElementGenerator._INSTANCE
                    .createIntersectionClass(mapping.getModel().getNamespace(
                            subject.getNamespace()), subject.getLocalName());
        else if (predicate.equals(OWL.UNION_OF_URI))
            expCls = OWLTypedElementGenerator._INSTANCE.createUnionClass(
                    mapping.getModel().getNamespace(subject.getNamespace()),
                    subject.getLocalName());
        else if (predicate.equals(OWL.COMPLEMENT_OF_URI))
            expCls = OWLTypedElementGenerator._INSTANCE.createComplementClass(
                    mapping.getModel().getNamespace(subject.getNamespace()),
                    subject.getLocalName());
        mapping.getModel().addClass(expCls);

        List axioms = repository.getReferences();
        for (int i = 0; i < axioms.size();) {
            ReferenceRecorder axiom = (ReferenceRecorder) axioms.get(i);
            if (!axiom.getReferedClass().getURI().equals(cls.getURI())) {
                i++;
                continue;
            }
            if (axiom.getProperty().getFullURI().equals(RDF.P_TYPE_STR)) {
                ((Individual) axiom.getReferingResource()).getRDFType().remove(
                        axiom.getReferedClass());
                ((Individual) axiom.getReferingResource()).getRDFType().add(
                        expCls);
                axioms.remove(i);
            }
        }
    }

    private void handleOneOf(RDFResourceElement subject,
            OWLModelDocRepository repository, OWLModelDocMapping mapping) {
        //subject is an OWL Data Range
        if (mapping.getModel().getDataRange(subject.getFullURI()) != null)
            return;
        //get the class in the current model
        OWLClass cls = mapping.getModel().getClass(subject.getFullURI());
        if (cls == null)
            throw new OWLParserException("Class ["
                                         + subject.getFullURI()
                                         + "] is not defined.");
        mapping.getModel().getClasses().remove(subject.getFullURI());
        mapping.getModel().getOntology().getContains().remove(cls);
        OWLClass enuCls = OWLTypedElementGenerator._INSTANCE
                .createEnumeratedClass(mapping.getModel().getNamespace(
                        subject.getNamespace()), subject.getLocalName());
        mapping.getModel().addClass(enuCls);

        List axioms = repository.getReferences();
        for (int i = 0; i < axioms.size();) {
            ReferenceRecorder axiom = (ReferenceRecorder) axioms.get(i);
            if (!axiom.getReferedClass().getURI().equals(cls.getURI())) {
                i++;
                continue;
            }
            if (axiom.getProperty().getFullURI().equals(RDF.P_TYPE_STR)) {
                ((Individual) axiom.getReferingResource()).getRDFType().remove(
                        axiom.getReferedClass());
                ((Individual) axiom.getReferingResource()).getRDFType().add(
                        enuCls);
                axioms.remove(i);
            }
        }
    }

    public OWLClass getClass(String namespace, String localName,
            OWLModelDocMapping mapping) {
        String clsURI = namespace + localName;
        OWLOntoModel ontoModel = mapping.getModel();
        OWLClass cls = ontoModel.getClass(clsURI);
        if (cls == null) {
            Map impModels = ontoModel.getImpModels();
            Iterator keys = impModels.keySet().iterator();
            while (keys.hasNext()) {
                cls = (OWLClass) ((OWLOntoModel) impModels.get(keys.next()))
                        .getClass(clsURI);
                if (cls != null)
                    break;
            }
        }
        if (cls == null)
            cls = getRestriction(namespace, localName, mapping);
        return cls;
    }

    public OWLRestriction getRestriction(String namespace, String localName,
            OWLModelDocMapping mapping) {
        String resURI = namespace + localName;
        OWLOntoModel ontoModel = mapping.getModel();
        OWLRestriction res = ontoModel.getRestriction(resURI);
        return res;
    }

    public OWLObjectProperty getObjectProperty(String namespace,
            String localName, OWLModelDocMapping mapping) {
        String proURI = namespace + localName;
        OWLOntoModel ontoModel = mapping.getModel();
        OWLObjectProperty objProperty = ontoModel.getObjectProperty(proURI);
        if (objProperty == null) {
            Map impModels = ontoModel.getImpModels();
            Iterator keys = impModels.keySet().iterator();
            while (keys.hasNext()) {
                objProperty = (OWLObjectProperty) ((OWLOntoModel) impModels
                        .get(keys.next())).getObjectProperty(proURI);
                if (objProperty != null)
                    break;
            }
        }
        return objProperty;
    }

    public OWLDatatypeProperty getDatatypeProperty(String namespace,
            String localName, OWLModelDocMapping mapping) {
        String proURI = namespace + localName;
        OWLOntoModel ontoModel = mapping.getModel();
        OWLDatatypeProperty dataProperty = ontoModel
                .getDatatypeProperty(proURI);
        if (dataProperty == null) {
            Map impModels = ontoModel.getImpModels();
            Iterator keys = impModels.keySet().iterator();
            while (keys.hasNext()) {
                dataProperty = (OWLDatatypeProperty) ((OWLOntoModel) impModels
                        .get(keys.next())).getDatatypeProperty(proURI);
                if (dataProperty != null)
                    break;
            }
        }
        return dataProperty;
    }

    public RDFSDatatype getDatatype(String namespace, String localName,
            OWLModelDocMapping mapping) {
        String typeURI = namespace + localName;
        OWLOntoModel ontoModel = mapping.getModel();
        RDFSDatatype datatype = ontoModel.getDatatype(typeURI);
        if (datatype == null) {
            Map impModels = ontoModel.getImpModels();
            Iterator keys = impModels.keySet().iterator();
            while (keys.hasNext()) {
                datatype = (RDFSDatatype) ((OWLOntoModel) impModels.get(keys
                        .next())).getDatatype(typeURI);
                if (datatype != null)
                    break;
            }
        }
        return datatype;
    }

    public Individual getIndividual(String namespace, String localName,
            OWLModelDocMapping mapping) {
        String invURI = namespace + localName;
        OWLOntoModel ontoModel = mapping.getModel();
        Individual individual = ontoModel.getIndividual(invURI);
        if (individual == null) {
            Map impModels = ontoModel.getImpModels();
            Iterator keys = impModels.keySet().iterator();
            while (keys.hasNext()) {
                individual = (Individual) ((OWLOntoModel) impModels.get(keys
                        .next())).getIndividual(invURI);
                if (individual != null)
                    break;
            }
        }
        return individual;
    }
}