/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: RDFClassPropertyAnalyser.java,v 1.1.1.1 2006/01/31 09:13:26 gxie Exp $
 */

package org.eclipse.eodm.rdf.resource.parser.wrapper;

import org.eclipse.eodm.rdf.resource.parser.element.RDFLiteralElement;
import org.eclipse.eodm.rdf.resource.parser.element.RDFResourceElement;
import org.eclipse.eodm.rdf.resource.parser.element.RDFValue;
import org.eclipse.eodm.rdf.resource.parser.element.URIReference;
import org.eclipse.eodm.rdf.resource.parser.exception.ParserException;
import org.eclipse.eodm.rdfs.RDFProperty;
import org.eclipse.eodm.rdfs.RDFSClass;
import org.eclipse.eodm.rdfs.RDFSFactory;
import org.eclipse.eodm.rdfs.RDFSLiteral;
import org.eclipse.eodm.rdfs.RDFSResource;
import org.eclipse.eodm.rdfs.RDFStatement;
import org.eclipse.eodm.rdfs.vocabulary.RDF;
import org.eclipse.eodm.rdfs.vocabulary.RDFS;

/**
 * RDFClassPropertyAnalyser Some properties require RDFSubject and RDFObject of
 * an implicitly decidable type, eg. rdfs:subClassOf, rdfs:subPropertyOf,
 * rdfs:domain, rdfs:range, the property analyser finds the existing resource if
 * it is present, or generate it if not.
 */
public final class RDFClassPropertyAnalyser {
    private RDFClassPropertyAnalyser() {
    }

    public static final RDFClassPropertyAnalyser _INSTANCE = new RDFClassPropertyAnalyser();

    /**
     * Handle rdf property: rdfs:subClassOf / rdfs:subPropertyOf / rdfs:domain /
     * rdfs:range / rdf:subject / rdf:predicate / rdf: object
     * 
     * @param subject
     *            the subject of the statement
     * @param predicate
     *            the predicate of the statement
     * @param object
     *            the object of the statement
     * @param analyser
     *            the triple analyser
     * @return true if the triple is handled
     */
    public boolean handleRDFProperty(RDFResourceElement subject,
            URIReference predicate, RDFValue object, RDFTripleAnalyser analyser) {
        boolean handled = handleRDFSSubClass(subject, predicate, object,
                analyser);

        if (!handled)
            handled = handleRDFSubProperty(subject, predicate, object, analyser);

        if (!handled)
            handled = handleRDFPropertyDomain(subject, predicate, object,
                    analyser);

        if (!handled)
            handled = handleRDFReification(subject, predicate, object, analyser);

        return handled;
    }

    //rdfs:subClassOf
    private boolean handleRDFSSubClass(RDFResourceElement subject,
            URIReference predicate, RDFValue object, RDFTripleAnalyser analyser) {
        if (!predicate.getFullURI().equals(RDFS.NAMESPACE + RDFS.P_SUBCLASSOF))
            return false;

        RDFSClass subClass = RDFTypeIdentifier._INSTANCE.constructRDFSClass(
                subject.getNamespace(), subject.getLocalName(), analyser);

        RDFSClass objClass = RDFTypeIdentifier._INSTANCE.constructRDFSClass(
                ((RDFResourceElement) object).getNamespace(),
                ((RDFResourceElement) object).getLocalName(), analyser);

        subClass.getRDFSSubClassOf().add(objClass);
        objClass.getSubClass().add(subClass);

        return true;
    }

    //rdfs:subPropertyOf
    private boolean handleRDFSubProperty(RDFResourceElement subject,
            URIReference predicate, RDFValue object, RDFTripleAnalyser analyser) {
        if (!predicate.getFullURI().equals(
                RDFS.NAMESPACE + RDFS.P_SUBPROPERTYOF))
            return false;

        RDFProperty subProperty = RDFTypeIdentifier._INSTANCE
                .constructRDFProperty(subject.getNamespace(), subject
                        .getLocalName(), analyser);

        RDFProperty objProperty = RDFTypeIdentifier._INSTANCE
                .constructRDFProperty(((RDFResourceElement) object)
                        .getNamespace(), ((RDFResourceElement) object)
                        .getLocalName(), analyser);

        subProperty.getRDFSSubPropertyOf().add(objProperty);
        objProperty.getSubProperty().add(subProperty);

        return true;
    }

    //rdfs:domain
    private boolean handleRDFPropertyDomain(RDFResourceElement subject,
            URIReference predicate, RDFValue object, RDFTripleAnalyser analyser) {
        if (!predicate.getFullURI().equals(RDFS.NAMESPACE + RDFS.P_DOMAIN))
            return false;

        RDFProperty property = RDFTypeIdentifier._INSTANCE
                .constructRDFProperty(subject.getNamespace(), subject
                        .getLocalName(), analyser);

        RDFSClass cls = RDFTypeIdentifier._INSTANCE.constructRDFSClass(
                ((RDFResourceElement) object).getNamespace(),
                ((RDFResourceElement) object).getLocalName(), analyser);

        property.getRDFSDomain().add(cls);
        cls.getOwnedProperty().add(property);

        return true;
    }

    //rdfs:range
    public boolean handleRDFPropertyRange(RDFResourceElement subject,
            URIReference predicate, RDFValue object, RDFTripleAnalyser analyser) {
        if (!predicate.getFullURI().equals(RDFS.NAMESPACE + RDFS.P_RANGE))
            return false;

        RDFProperty property = RDFTypeIdentifier._INSTANCE
                .constructRDFProperty(subject.getNamespace(), subject
                        .getLocalName(), analyser);

        if ( !(object instanceof URIReference) )
            throw new ParserException("Invalid rdfs:range object");
        RDFResourceElement objResource = (RDFResourceElement) object;
        RDFSClass cls = analyser.getRepository().getDatatype(
                objResource.getNamespace() + objResource.getLocalName());
        if (cls == null)
            cls = RDFTypeIdentifier._INSTANCE.constructRDFSClass(objResource
                    .getNamespace(), objResource.getLocalName(), analyser);

        property.getRDFSRange().add(cls);
        cls.getRangedByProperty().add(property);

        return true;
    }

    //rdf:subject rdf:predicate rdf:object
    private boolean handleRDFReification(RDFResourceElement subject,
            URIReference predicate, RDFValue object, RDFTripleAnalyser analyser) {
        if (!predicate.getFullURI().equals(RDF.NAMESPACE + RDF.P_SUBJECT)
            && !predicate.getFullURI().equals(RDF.NAMESPACE + RDF.P_PREDICATE)
            && !predicate.getFullURI().equals(RDF.NAMESPACE + RDF.P_OBJECT))
            return false;

        //1. subject to be a statement
        RDFStatement statement = RDFTypeIdentifier._INSTANCE
                .constructRDFRefiedStatement(subject.getNamespace(), subject
                        .getLocalName(), analyser);

        statement.setReified(true);

        //2. object can be an RDFResourceElement or a literal
        //precreated a null resource with just uri
        RDFSResource objRes = null;
        if (object instanceof RDFResourceElement) {
            objRes = RDFSFactory.eINSTANCE.createRDFSResource();
            objRes.setNamespace(analyser
                    .getNamespaceByURI(((RDFResourceElement) object)
                            .getNamespace()));
            objRes.setLocalName(((RDFResourceElement) object).getLocalName());
        } else if (object instanceof RDFLiteralElement) {
            RDFSLiteral literal = analyser
                    .createLiteral((RDFLiteralElement) object);
            objRes = literal;
        }

        //3. predicate {rdf:subject / rdf:predicate / rdf:object }
        if (predicate.getLocalName().equals(RDF.P_SUBJECT))
            statement.setRDFSubject(objRes);
        if (predicate.getLocalName().equals(RDF.P_PREDICATE))
            statement.setRDFPredicate(objRes);
        if (predicate.getLocalName().equals(RDF.P_OBJECT))
            statement.setRDFObject(objRes);

        return true;
    }
}