/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: RDFTypeIdentifier.java,v 1.1.1.1 2006/01/31 09:13:26 gxie Exp $
 */

package org.eclipse.eodm.rdf.resource.parser.wrapper;

import java.util.Iterator;

import org.eclipse.eodm.rdf.resource.parser.element.RDFResourceElement;
import org.eclipse.eodm.rdf.resource.parser.element.RDFValue;
import org.eclipse.eodm.rdf.resource.parser.element.URIReference;
import org.eclipse.eodm.rdfs.RDFAlt;
import org.eclipse.eodm.rdfs.RDFBag;
import org.eclipse.eodm.rdfs.RDFList;
import org.eclipse.eodm.rdfs.RDFProperty;
import org.eclipse.eodm.rdfs.RDFSClass;
import org.eclipse.eodm.rdfs.RDFSContainer;
import org.eclipse.eodm.rdfs.RDFSDatatype;
import org.eclipse.eodm.rdfs.RDFSResource;
import org.eclipse.eodm.rdfs.RDFSeq;
import org.eclipse.eodm.rdfs.RDFStatement;
import org.eclipse.eodm.rdfs.vocabulary.RDF;
import org.eclipse.eodm.rdfs.vocabulary.RDFS;

/**
 * RDFTypeIdentifier identify all typed triples
 */
public final class RDFTypeIdentifier {
    private RDFTypeIdentifier() {
    }

    /** The order in which to find the resource generated before */
    public static final int[] RESOURCE_SEARCH_ORDER = new int[] {
            RDFResourceMap.RESOURCE, RDFResourceMap.CLASS,
            RDFResourceMap.PROPERTY, RDFResourceMap.CONTAINER,
            RDFResourceMap.LIST, RDFResourceMap.REFIED_STATEMENT,
            RDFResourceMap.DATATYPE };

    /** The single RDFTypeIdentifier instance */
    public static final RDFTypeIdentifier _INSTANCE = new RDFTypeIdentifier();

    /**
     * handle all <subject rdf:type object> triples, that's to say, the
     * predicate is equivalent to rdf:type
     * 
     * @param subject
     *            the subject of the triple
     * @param predicate
     *            the predicate of the triple
     * @param object
     *            the object of the triple
     * @param tripleAnalyser
     *            the triple analyser
     * @return true if the triple is handled
     */
    public boolean identifyRDFType(RDFResourceElement subject,
            URIReference predicate, RDFValue object,
            RDFTripleAnalyser tripleAnalyser) {
        if (!predicate.getFullURI().equals(RDF.P_TYPE_STR))
            return false;

        /* the object must be a rdf resource since the predicate is rdf:type */
        RDFResourceElement objRef = (RDFResourceElement) object;

        /* object type */
        int type = RDFTypeConstants.getRDFElementType(objRef.getFullURI());
        switch (type) {
        case RDFTypeConstants.RDF_ALT:
            constructRDFAlt(subject.getNamespace(), subject.getLocalName(),
                    tripleAnalyser);
            break;
        case RDFTypeConstants.RDF_BAG:
            constructRDFBag(subject.getNamespace(), subject.getLocalName(),
                    tripleAnalyser);
            break;
        case RDFTypeConstants.RDF_SEQ:
            constructRDFSeq(subject.getNamespace(), subject.getLocalName(),
                    tripleAnalyser);
            break;
        case RDFTypeConstants.RDF_PROPERTY:
            constructRDFProperty(subject.getNamespace(),
                    subject.getLocalName(), tripleAnalyser);
            break;
        case RDFTypeConstants.RDF_STATEMENT:
            constructRDFRefiedStatement(subject.getNamespace(), subject
                    .getLocalName(), tripleAnalyser);
            break;
        case RDFTypeConstants.RDF_XMLLITERAL:
            break;
        case RDFTypeConstants.RDF_LIST:
            constructRDFList(subject.getNamespace(), subject.getLocalName(),
                    tripleAnalyser);
            break;
        case RDFTypeConstants.RDFS_CLASS:
            constructRDFSClass(subject.getNamespace(), subject.getLocalName(),
                    tripleAnalyser);
            break;
        case RDFTypeConstants.RDFS_CONTAINER:
            constructRDFSContainer(subject.getNamespace(), subject
                    .getLocalName(), tripleAnalyser);
            break;
        case RDFTypeConstants.RDFS_CONTAINER_MEMEBERSHIP_PROPERTY:
            break;
        case RDFTypeConstants.RDFS_DATATYPE:
            constructRDFSDatatype(subject.getNamespace(), subject
                    .getLocalName(), tripleAnalyser);
            break;
        case RDFTypeConstants.RDFS_LITERAL:
            break;
        case RDFTypeConstants.RDFS_RESOURCE:
            constructRDFSResource(subject.getNamespace(), subject
                    .getLocalName(), tripleAnalyser);
            break;
        default:
            //indicating that the type is defined in the document, so it will
            // be pregenerated
            RDFSResource resource = constructRDFSResource(subject
                    .getNamespace(), subject.getLocalName(), tripleAnalyser);
            RDFSClass typeCls = constructRDFSClass(
                    ((RDFResourceElement) object).getNamespace(),
                    ((RDFResourceElement) object).getLocalName(),
                    tripleAnalyser);
            resource.getRDFType().add(typeCls);
            break;
        }

        return true;
    }

    /**
     * Construct an rdf list if it is absent from the repository
     * 
     * @param namespace
     *            the list namespace
     * @param localName
     *            the list local name
     * @param analyser
     *            the triple analyser
     * @return the rdf list
     */
    public RDFList constructRDFList(String namespace, String localName,
            RDFTripleAnalyser analyser) {
        RDFList list = analyser.getRepository().getList(namespace + localName);
        if (list == null) {
            list = RDFTypeGenerator._INSTANCE.createList(analyser
                    .getNamespaceByURI(namespace), localName);
            analyser.getDescription().getContains().add(list);
            analyser.getRepository().addList(list);
        }
        return list;
    }

    /**
     * Construct an rdfs datatype if it is absent from the repository
     * 
     * @param namespace
     *            the datatype namespace
     * @param localName
     *            the datatype local name
     * @param analyser
     *            the triple analyser
     * @return the datatype
     */
    public RDFSDatatype constructRDFSDatatype(String namespace,
            String localName, RDFTripleAnalyser analyser) {
        RDFSDatatype datatype = analyser.getRepository().getDatatype(
                namespace + localName);
        if (datatype == null) {
            datatype = RDFTypeGenerator._INSTANCE.createDatatype(analyser
                    .getNamespaceByURI(namespace), localName);
            analyser.getDescription().getContains().add(datatype);
            analyser.getRepository().addDatatype(datatype);
        }
        return datatype;
    }

    /**
     * Construct an rdf alt container if it is absent from the repository. Note:
     * If the container is present with a super type rdfs:Container, it has to
     * be retyped as an alt container.
     * 
     * @param namespace
     *            the alt container namespace
     * @param localName
     *            the alt container local name
     * @param analyser
     *            the triple analyser
     * @return the alt container
     */
    public RDFSContainer constructRDFAlt(String namespace, String localName,
            RDFTripleAnalyser analyser) {
        RDFSContainer container = analyser.getRepository().getContainer(
                namespace + localName);
        RDFAlt alt = null;
        if (container == null || !(container instanceof RDFAlt)) {
            alt = RDFTypeGenerator._INSTANCE.createAlt(analyser
                    .getNamespaceByURI(namespace), localName);
            analyser.getDescription().getContains().add(alt);
            analyser.getRepository().addContainer(alt);
        } else
            alt = (RDFAlt) container;

        if (container != null && !(container instanceof RDFAlt)) {
            Iterator members = container.getRDFSMember().iterator();
            while (members.hasNext())
                alt.getRDFSMember().add(members.next());
            analyser.getRepository().removeContainer(namespace + localName);
            analyser.getRepository().addContainer(alt);
        }
        return alt;
    }

    /**
     * Construct an rdf bag container if it is absent from the repository. Note:
     * If the container is present with a super type rdfs:Container, it has to
     * be retyped as a bag container.
     * 
     * @param namespace
     *            the bag container namespace
     * @param localName
     *            the bag container local name
     * @param analyser
     *            the triple analyser
     * @return the bag container
     */
    public RDFSContainer constructRDFBag(String namespace, String localName,
            RDFTripleAnalyser analyser) {
        RDFSContainer container = analyser.getRepository().getContainer(
                namespace + localName);
        RDFBag bag = null;
        if (container == null || !(container instanceof RDFBag)) {
            bag = RDFTypeGenerator._INSTANCE.createBag(analyser
                    .getNamespaceByURI(namespace), localName);
            analyser.getDescription().getContains().add(bag);
            analyser.getRepository().addContainer(bag);
        } else
            bag = (RDFBag) container;

        if (container != null && !(container instanceof RDFBag)) {
            Iterator members = container.getRDFSMember().iterator();
            while (members.hasNext())
                bag.getRDFSMember().add(members.next());
            analyser.getRepository().removeContainer(namespace + localName);
            analyser.getRepository().addContainer(bag);
        }
        return bag;
    }

    /**
     * Construct an rdf seq container if it is absent from the repository. Note:
     * If the container is present with a super type rdfs:Container, it has to
     * be retyped as an seq container.
     * 
     * @param namespace
     *            the seq container namespace
     * @param localName
     *            the seq container local name
     * @param analyser
     *            the triple analyser
     * @return the seq container
     */
    public RDFSContainer constructRDFSeq(String namespace, String localName,
            RDFTripleAnalyser analyser) {
        RDFSContainer container = analyser.getRepository().getContainer(
                namespace + localName);
        RDFSeq seq = null;
        if (container == null || !(container instanceof RDFSeq)) {
            seq = RDFTypeGenerator._INSTANCE.createSeq(analyser
                    .getNamespaceByURI(namespace), localName);
            analyser.getDescription().getContains().add(seq);
            analyser.getRepository().addContainer(seq);
        } else
            seq = (RDFSeq) container;

        if (container != null && !(container instanceof RDFSeq)) {
            Iterator members = container.getRDFSMember().iterator();
            while (members.hasNext())
                seq.getRDFSMember().add(members.next());
            analyser.getRepository().removeContainer(namespace + localName);
            analyser.getRepository().addContainer(seq);
        }
        return seq;
    }

    /**
     * Construct an rdfs container: If it is not present, create it. Or just get
     * it and return it.
     * 
     * @param namespace
     *            the container namespace
     * @param localName
     *            the container local name
     * @param analyser
     *            the triple analyser
     * @return the container
     */
    public RDFSContainer constructRDFSContainer(String namespace,
            String localName, RDFTripleAnalyser analyser) {
        RDFSContainer container = analyser.getRepository().getContainer(
                namespace + localName);
        if (container == null) {
            container = RDFTypeGenerator._INSTANCE.createContainer(analyser
                    .getNamespaceByURI(namespace), localName);
            analyser.getDescription().getContains().add(container);
            analyser.getRepository().addContainer(container);
        }
        return container;
    }

    /**
     * Construct an rdf property resource with the namespace and local name
     * 
     * @param namespace
     *            the property namespace
     * @param localName
     *            the property local name
     * @param analyser
     *            the triple analyser
     * @return the rdf property
     */
    public RDFProperty constructRDFProperty(String namespace, String localName,
            RDFTripleAnalyser analyser) {
        RDFProperty property = analyser.getRepository().getProperty(
                namespace + localName);
        if (property == null) {
            property = RDFTypeGenerator._INSTANCE.createProperty(analyser
                    .getNamespaceByURI(namespace), localName);
            analyser.getDescription().getContains().add(property);
            analyser.getRepository().addProperty(property);
        }
        return property;
    }

    /**
     * Construct an rdf reification statement with a null subject, predicate,
     * object
     * 
     * @param namespace
     *            the statement namespace
     * @param localName
     *            the statement local name
     * @param analyser
     *            the triple analyser
     * @return the rdf statement
     */
    public RDFStatement constructRDFRefiedStatement(String namespace,
            String localName, RDFTripleAnalyser analyser) {
        RDFStatement statement = analyser.getRepository().getRefiedStatement(
                namespace + localName);
        if (statement == null) {
            statement = RDFTypeGenerator._INSTANCE.createStatement(analyser
                    .getNamespaceByURI(namespace), localName);
            analyser.getDescription().getContains().add(statement);
            analyser.getRepository().addRefiedStatement(statement);
        }
        return statement;
    }

    /**
     * Construct an rdf resource
     * 
     * @param namespace
     *            the resource namespace
     * @param localName
     *            the resource local name
     * @param analyser
     *            the triple analyser
     * @return the rdf resource
     */
    public RDFSResource constructRDFSResource(String namespace,
            String localName, RDFTripleAnalyser analyser) {
        RDFSResource resource = analyser.getRepository().getGlobalResource(
                namespace + localName, RESOURCE_SEARCH_ORDER);
        if (resource == null) {
            resource = RDFTypeGenerator._INSTANCE.createResource(analyser
                    .getNamespaceByURI(namespace), localName);
            analyser.getDescription().getContains().add(resource);
            analyser.getRepository().addResource(resource);
        }
        return resource;
    }

    /**
     * Construct an rdf class: If the namespace parameter is euivalent to XSD
     * namespace, a datatype(a sub class of RDFSClass) is generated. Or an rdf
     * class is created as the result.
     * 
     * @param namespace
     *            the class namespace
     * @param localName
     *            the class local name
     * @param analyser
     *            the triple analyser
     * @return the rdfs class or the rdfs datatype
     */
    public RDFSClass constructRDFSClass(String namespace, String localName,
            RDFTripleAnalyser analyser) {
        if (namespace.equals(RDFS.XSD_NAMESPACE))
            return constructRDFSDatatype(namespace, localName, analyser);
        else {
            RDFSClass cls = analyser.getRepository().getClass(
                    namespace + localName);
            if (cls == null) {
                cls = RDFTypeGenerator._INSTANCE.createClass(analyser
                        .getNamespaceByURI(namespace), localName);
                analyser.getRepository().addClass(cls);
                analyser.getDescription().getContains().add(cls);
            }
            return cls;
        }
    }
}