/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: RDFUtilityPropertyAnalyser.java,v 1.1.1.1 2006/01/31 09:13:26 gxie Exp $
 */

package org.eclipse.eodm.rdf.resource.parser.wrapper;

import org.eclipse.eodm.rdf.resource.parser.element.RDFLiteralElement;
import org.eclipse.eodm.rdf.resource.parser.element.RDFResourceElement;
import org.eclipse.eodm.rdf.resource.parser.element.RDFValue;
import org.eclipse.eodm.rdf.resource.parser.element.URIReference;
import org.eclipse.eodm.rdfs.PlainLiteral;
import org.eclipse.eodm.rdfs.RDFSFactory;
import org.eclipse.eodm.rdfs.RDFSResource;
import org.eclipse.eodm.rdfs.vocabulary.RDF;
import org.eclipse.eodm.rdfs.vocabulary.RDFS;

/**
 * RDFUtilityPropertyAnalyser Analyse some utility property triples: rdf:value
 * rdf:label rdf:comment rdfs:member rdfs:isDefinedBy rdfs:seeAlso
 */
public final class RDFUtilityPropertyAnalyser {
    private RDFUtilityPropertyAnalyser() {
    }

    /** the single instance of class RDFUtilityPropertyAnalyser */
    public static final RDFUtilityPropertyAnalyser _INSTANCE = new RDFUtilityPropertyAnalyser();

    /** The order in which to find the resource generated before */
    public static final int[] RESOURCE_SEARCH_ORDER = new int[] {
            RDFResourceMap.RESOURCE, RDFResourceMap.CLASS,
            RDFResourceMap.PROPERTY, RDFResourceMap.REFIED_STATEMENT,
            RDFResourceMap.CONTAINER, RDFResourceMap.LIST,
            RDFResourceMap.DATATYPE };

    /**
     * To handle such triples with predicate of: rdf:value rdf:label rdf:comment
     * rdfs:member rdfs:isDefinedBy rdfs:seeAlso
     * 
     * @param subject
     *            the subject of the triple
     * @param predicate
     *            the predicate of the triple
     * @param object
     *            the object of the triple
     * @param analyser
     *            the triple analyser
     * @return true if the predicate is identified as one of those URIreferences
     *         above
     */
    public boolean handleRDFUtilityTriple(RDFResourceElement subject,
            URIReference predicate, RDFValue object, RDFTripleAnalyser analyser) {
        if (!predicate.getFullURI().equals(RDF.NAMESPACE + RDF.P_VALUE)
            && !predicate.getFullURI().equals(RDFS.NAMESPACE + RDFS.P_LABEL)
            && !predicate.getFullURI().equals(RDFS.NAMESPACE + RDFS.P_COMMENT)
            && !predicate.getFullURI().equals(RDFS.NAMESPACE + RDFS.MEMBER)
            && !predicate.getFullURI().equals(RDFS.NAMESPACE + RDFS.SEEALSO)
            && !predicate.getFullURI()
                    .equals(RDFS.NAMESPACE + RDFS.ISDEFINEDBY))
            return false;

        RDFSResource subRes = checkExistingResource(subject, analyser);
        RDFSResource objRes = null;

        //rdf:value
        if (predicate.getFullURI().equals(RDF.NAMESPACE + RDF.P_VALUE)) {
            objRes = checkExistingResource(object, analyser);
            subRes.getRDFValue().add(objRes);
            return true;
        }

        //rdfs:label <PlainLiteral>
        if (predicate.getFullURI().equals(RDFS.NAMESPACE + RDFS.P_LABEL)) {
            PlainLiteral literal = RDFSFactory.eINSTANCE.createPlainLiteral();
            literal.setLanguage(((RDFLiteralElement) object).getLanguage());
            literal.setLexicalForm(((RDFLiteralElement) object).getText());            
            analyser.getDescription().getContains().add(literal);
            subRes.getRDFSLabel().add(literal);
            return true;
        }

        //rdfs:comment <PlainLiteral>
        if (predicate.getFullURI().equals(RDFS.NAMESPACE + RDFS.P_COMMENT)) {
            PlainLiteral literal = RDFSFactory.eINSTANCE.createPlainLiteral();
            literal.setLanguage(((RDFLiteralElement) object).getLanguage());
            literal.setLexicalForm(((RDFLiteralElement) object).getText());
            analyser.getDescription().getContains().add(literal);
            subRes.getRDFSComment().add(literal);
            return true;
        }

        //rdfs:member
        if (predicate.getFullURI().equals(RDFS.NAMESPACE + RDFS.MEMBER)) {
            objRes = checkExistingResource(object, analyser);
            subRes.getRDFSMember().add(objRes);
            return true;
        }

        //rdfs:seeAlso
        if (predicate.getFullURI().equals(RDFS.NAMESPACE + RDFS.SEEALSO)) {
            objRes = checkExistingResource(object, analyser);
            subRes.getRDFSSeeAlso().add(objRes);
            return true;
        }

        //rdfs:isDefinedBy
        if (predicate.getFullURI().equals(RDFS.NAMESPACE + RDFS.ISDEFINEDBY)) {
            objRes = checkExistingResource(object, analyser);
            subRes.getRDFSIsDefinedBy().add(objRes);
            return true;
        }

        return true;
    }

    /**
     * To check whether the designated original value exists in the repository,
     * if not, it will be generated.
     * 
     * @param original
     *            the original value
     * @param analyser
     *            the triple analyser
     * @return the rdf resource
     */
    private RDFSResource checkExistingResource(RDFValue original,
            RDFTripleAnalyser analyser) {
        if (original == null)
            return null;

        if (original instanceof RDFLiteralElement)
            return analyser.createLiteral((RDFLiteralElement) original);

        RDFResourceElement originalResource = (RDFResourceElement) original;
        return RDFTypeIdentifier._INSTANCE.constructRDFSResource(
                originalResource.getNamespace(), originalResource
                        .getLocalName(), analyser);
    }
}