/**
 * <copyright> 
 * 
 * Copyright (c) 2004-2005 IBM Corporation and others. 
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License - v 1.0 
 * which accompanies this distribution, and is available at 
 * http://opensource.org/licenses/eclipse-1.0.txt 
 * 
 * Contributors: 
 *   IBM - Initial API and implementation 
 * 
 * </copyright> 
 * 
 * $Id: ECore2RDFSTransformer.java,v 1.2 2006/06/22 05:07:57 gxie Exp $
 */

package org.eclipse.eodm.rdfs.transformer.ecore;

import java.util.HashMap;
import java.util.Iterator;

import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EAnnotation;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EDataType;
import org.eclipse.emf.ecore.EEnum;
import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.ENamedElement;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EcorePackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.eodm.rdf.resource.RDFXMLResource;
import org.eclipse.eodm.rdf.resource.RDFXMLResourceFactoryImpl;
import org.eclipse.eodm.rdfs.Namespace;
import org.eclipse.eodm.rdfs.Ontology;
import org.eclipse.eodm.rdfs.PlainLiteral;
import org.eclipse.eodm.rdfs.RDFBag;
import org.eclipse.eodm.rdfs.RDFProperty;
import org.eclipse.eodm.rdfs.RDFSClass;
import org.eclipse.eodm.rdfs.RDFSDatatype;
import org.eclipse.eodm.rdfs.RDFSFactory;
import org.eclipse.eodm.rdfs.RDFSLiteral;
import org.eclipse.eodm.rdfs.RDFSResource;
import org.eclipse.eodm.rdfs.transformer.RDFSTransformationException;
import org.eclipse.eodm.rdfs.vocabulary.XSD;

/**
 * 
 * The API realizes translation from ECore model to RDF model.
 * 
 *  
 */
public class ECore2RDFSTransformer {

    //	input file
    private String inputEcoreFile = null;

    //  output file
    private String outputRDFFile = null;

    private RDFSFactory factory = null;

    // RDFS ontology
    private Ontology ontology = null;

    //Ecore
    //private EcoreFactory ecoreFactory = null;

    private EcorePackage ecorePackage = null;

    //Ecore pacakge
    //private EPackage epackage = null;

    private ResourceSet resourceSet = null;

    public ECore2RDFSTransformer() {

        factory = RDFSFactory.eINSTANCE;
        //create ontology
        ontology = factory.createOntology();

        //init Ecore package
        //ecoreFactory = EcoreFactory.eINSTANCE;
        ecorePackage = EcorePackage.eINSTANCE;
        //epackage = ecoreFactory.createEPackage();
        resourceSet = new ResourceSetImpl();
        resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
                .put(Resource.Factory.Registry.DEFAULT_EXTENSION,
                        new XMIResourceFactoryImpl());
    }

    /**
     * 
     * Translate function Translate the input ecore file to target rdfs file.
     * 
     * @param input
     * @throws RDFSTransformationException
     */
    public void convertECore2RDFS(String input) throws RDFSTransformationException {
        convertECore2RDFS(input, null);
    }

    /**
     * Convert input ecore model
     * 
     * @param resourceSet
     *            input ecore model to RDF file
     * @param output
     *            target rdfs file
     *  
     */
    public void convertECore2RDFS(ResourceSet resSet, String output)
            throws RDFSTransformationException {
        if (output != null) {
            if (output.equals("") == false) {
                setOutputRDFFile(output);
            }
        }

        resourceSet = resSet;
        //load ROOT EPackage
        Resource resource = (Resource)resourceSet.getResources().get(0);

        EPackage ep = null;
        if ((resource.getContents().size() == 0)
            || !EPackage.class.isInstance(resource.getContents().get(0))) {
            RDFSTransformationException e = new RDFSTransformationException(
                    "Input Ecore has no ROOT EPackage : input Ecore model is invalid.");
            throw e;
        } else {
            ep = (EPackage) resource.getContents().get(0);
        }

        try {
            // EPackage -> RDFDescription

            //add namespace
            Namespace ns = factory.createNamespace();
            ns.setName(ep.getNsPrefix());
            ns.setURI((ep.getNsURI().endsWith("#") ? ep.getNsURI() : (ep
                    .getNsURI() + "#")));
            ontology.getOwnedNamespace().add(ns);

            //Ecore -> RDFS

            //create ontology from EPackage
            PlainLiteral label = factory.createPlainLiteral();
            label.setLexicalForm("EPackage_" + ep.getName());
            ontology.getRDFSLabel().add(label);
            ontology.setLocalName(ep.getName());
            ontology.setNamespace(ns);
            createComments(ontology, ep, ontology);
            Namespace xsd = null;

            for (Iterator iter = ep.getEClassifiers().iterator(); iter
                    .hasNext();) {

                EClassifier classifier = (EClassifier) iter.next();

                //EClass -> RDFSClass
                if (classifier instanceof EClass) {
                    // create RDFSClass from EClass
                    EClass eclass = (EClass) classifier;
                    RDFSClass rclass = factory.createRDFSClass();
                    rclass.setLocalName(eclass.getName());
                    rclass.setNamespace(ns);

                    createComments(ontology, eclass, rclass);
                    label = factory.createPlainLiteral();
                    label.setLexicalForm("EClass_" + eclass.getName());
                    rclass.getRDFSLabel().add(label);
                    ontology.getContains().add(rclass);
                }
                //EEnum -> RDFBag
                else if (classifier instanceof EEnum) {

                    EEnum eenum = (EEnum) classifier;

                    RDFBag rbag = factory.createRDFBag();
                    rbag.setLocalName(eenum.getName());
                    rbag.setNamespace(ns);

                    label = factory.createPlainLiteral();
                    label.setLexicalForm("EEnum_" + eenum.getName());
                    rbag.getRDFSLabel().add(label);
                    createComments(ontology, eenum, rbag);

                    //EEnumLiteral -> RDFSMember
                    Iterator et = eenum.getELiterals().iterator();
                    while (et.hasNext()) {
                        EEnumLiteral enumLiteral = (EEnumLiteral) et.next();
                        RDFSLiteral literal = factory.createPlainLiteral();
                        literal.setLexicalForm(enumLiteral.getName());
                        rbag.getRDFSMember().add(literal);
                    }
                    ontology.getContains().add(rbag);
                }
                //EDataType -> RDFSDatatype
                else if (classifier instanceof EDataType) {
                    EDataType datatype = (EDataType) classifier;
                    RDFSDatatype rdatatype = factory.createRDFSDatatype();
                    rdatatype.setLocalName(datatype.getName());
                    rdatatype.setNamespace(ns);

                    label = factory.createPlainLiteral();
                    label.setLexicalForm("EDatatype_" + datatype.getName());
                    rdatatype.getRDFSLabel().add(label);
                    createComments(ontology, datatype, rdatatype);

                    ontology.getContains().add(rdatatype);
                }
            }

            //eSuperTypes -> RDFSSubClassOf
            for (Iterator iter = ep.getEClassifiers().iterator(); iter
                    .hasNext();) {
                EClassifier classifier = (EClassifier) iter.next();
                if (classifier instanceof EClass) {
                    EClass eclass = (EClass) classifier;
                    Iterator sulist = eclass.getESuperTypes().iterator();
                    RDFSClass rclass = (RDFSClass) ontology
                            .getContainedResource(null, eclass.getName());
                    if (rclass == null)
                        continue;

                    while (sulist.hasNext()) {
                        EClassifier temps = (EClassifier) sulist.next();
                        if (temps instanceof EClass) {
                            RDFSClass father = (RDFSClass) ontology
                                    .getContainedResource(null, temps.getName());
                            if (father == null)
                                continue;
                            rclass.getRDFSSubClassOf().add(father);
                        }
                    }
                }
            }

            //EAttribute & EReference -> RDFProperty
            for (Iterator iter = ep.getEClassifiers().iterator(); iter
                    .hasNext();) {
                EClassifier classifier = (EClassifier) iter.next();
                if (classifier instanceof EClass) {

                    EClass eclass = (EClass) classifier;
                    RDFSClass rclass = (RDFSClass) ontology
                            .getContainedResource(null, eclass.getName());
                    if (rclass == null)
                        continue;

                    //EAttribute -> RDFProperty
                    Iterator i = eclass.getEAttributes().iterator();
                    while (i.hasNext()) {
                        // If existing property
                        boolean existingProperty = false;
                        EAttribute attribute = (EAttribute) i.next();

                        RDFProperty rap = (RDFProperty) ontology
                                .getContainedResource(null, attribute.getName());
                        if (rap == null) {

                            // New property
                            rap = factory.createRDFProperty();
                            rap.setLocalName(attribute.getName());
                            rap.setNamespace(ns);

                            existingProperty = true;
                            ontology.getContains().add(rap);
                            createComments(ontology, attribute, rap);
                        }
                        rap.getRDFSDomain().add(rclass);

                        if (!EEnum.class.isInstance(attribute.getEType())) {
                            //create range as a datatype instance, including
                            // XSD, EMF and user-defined datatype

                            RDFSDatatype type = factory.createRDFSDatatype();
                            //check if XSD datatype
                            if (attribute.getEType() == ecorePackage
                                    .getEBoolean())
                                type.setLocalName(XSD.sxboolean);
                            else if (attribute.getEType() == ecorePackage
                                    .getEFloat())
                                type.setLocalName(XSD.sxfloat);
                            else if (attribute.getEType() == ecorePackage
                                    .getEByte())
                                type.setLocalName(XSD.sxbyte);
                            else if (attribute.getEType() == ecorePackage
                                    .getEInt())
                                type.setLocalName(XSD.sxint);
                            else if (attribute.getEType() == ecorePackage
                                    .getELong())
                                type.setLocalName(XSD.sxlong);
                            else if (attribute.getEType() == ecorePackage
                                    .getEDouble())
                                type.setLocalName(XSD.sxdouble);
                            else if (attribute.getEType() == ecorePackage
                                    .getEShort())
                                type.setLocalName(XSD.sxshort);
                            else if (attribute.getEType() == ecorePackage
                                    .getEIntegerObject())
                                type.setLocalName(XSD.sinteger);
                            else if (attribute.getEType() == ecorePackage
                                    .getEString())
                                type.setLocalName(XSD.sxstring);

                            if (type.getLocalName() != null) {
                                //is XSD type
                                if (xsd == null) {
                                    xsd = factory.createNamespace();
                                    xsd.setName(XSD.SHORTNAME);
                                    xsd.setURI(XSD.NAMESPACE);
                                    ontology.getOwnedNamespace().add(xsd);
                                }
                                type.setNamespace(xsd);
                                if (!existingProperty) {
                                    // Create label
                                    label = factory.createPlainLiteral();
                                    label.setLexicalForm("EAttribute_"
                                                         + attribute.getName());
                                    rap.getRDFSLabel().add(label);
                                }
                            } else {
                                //is user-defined datatype
                                type = null;
                                type = (RDFSDatatype) ontology
                                        .getContainedResource(null, attribute
                                                .getEType().getName());
                            }
                            addRDFSRange(rap, type);
                        } else {
                            // a bag attribute
                            RDFBag bag = (RDFBag) ontology
                                    .getContainedResource(null, attribute
                                            .getEType().getName());
                            if (!existingProperty) {
                                label = factory.createPlainLiteral();
                                label.setLexicalForm("EAttribute_EEnum_"
                                                     + attribute.getName());
                                rap.getRDFSLabel().add(label);
                            }
                            if (bag != null) {
                                RDFSDatatype dummy = factory
                                        .createRDFSDatatype();
                                dummy.setNamespace(ns);
                                dummy.setLocalName("Bag_" + bag.getLocalName());
                                dummy.getRDFSIsDefinedBy().add(bag);
                                addRDFSRange(rap, dummy);
                                ontology.getContains().add(dummy);
                            }
                        }
                    }

                    //EReference -> RDFProperty
                    i = eclass.getEReferences().iterator();
                    while (i.hasNext()) {

                        //boolean existingProperty = false;

                        EReference reference = (EReference) i.next();

                        RDFProperty rap = (RDFProperty) ontology
                                .getContainedResource(null, reference.getName());

                        if (rap == null) {

                            // New RDFProperty
                            rap = factory.createRDFProperty();
                            rap.setLocalName(reference.getName());
                            rap.setNamespace(ns);
                            ontology.getContains().add(rap);

                            //existingProperty = true;

                            // Create label
                            label = factory.createPlainLiteral();
                            label.setLexicalForm("EReference_"
                                                 + reference
                                                         .getEReferenceType()
                                                         .getName());
                            rap.getRDFSLabel().add(label);
                            createComments(ontology, reference, rap);
                        }
                        rap.getRDFSDomain().add(rclass);
                        RDFSClass range = (RDFSClass) ontology
                                .getContainedResource(null, reference
                                        .getEReferenceType().getName());
                        if (range != null)
                            addRDFSRange(rap, range);
                    }

                }

            }
        } catch (Exception e) {
            e.printStackTrace();
            RDFSTransformationException te = new RDFSTransformationException(
                    "Ecore to RDFS Mapping failed: " + e.toString());
            throw te;
        }

        try {
            resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
                    .put(Resource.Factory.Registry.DEFAULT_EXTENSION,
                            new RDFXMLResourceFactoryImpl());
            Resource saver = resourceSet.createResource(URI
                    .createFileURI(outputRDFFile));
            saver.getContents().add(ontology);
            HashMap options = new HashMap();
            options.put(RDFXMLResource.OPTION_ENCODING, "UTF-8");
            saver.save(options);
        } catch (Exception e) {
            RDFSTransformationException te = new RDFSTransformationException(
                    "Error during serialize into rdfs file: " + e.toString());
            throw te;
        }

    }

    /**
     * 
     * Translate function Translate the input ecore file to target rdfs file.
     * 
     * @param input
     *            input ecore file
     * @param output
     *            target rdfs file
     * @throws ModelException
     */
    public void convertECore2RDFS(String input, String output)
            throws RDFSTransformationException {

        //check input params
        if (input == null) {
            RDFSTransformationException e = new RDFSTransformationException(
                    "Initate mapping failed : input Ecore model is NULL.");
            throw e;
        }

        //check input and output files
        if (input != null)
            if (input.equals("") == false)
                setInputEcoreFile(input);
        if (output != null)
            if (output.equals("") == false)
                setOutputRDFFile(output);

        //load ROOT EPackage
        resourceSet.getResource(URI.createFileURI(inputEcoreFile), true);

        convertECore2RDFS(resourceSet,output);
    }

    /**
     * Add range to property without duplication
     * 
     * @param property
     * @param range
     */
    private void addRDFSRange(RDFProperty property, RDFSClass range) {
        if (property == null || range == null)
            return;

        boolean existingRange = false;
        for (Iterator iter = property.getRDFSRange().iterator(); iter.hasNext();) {
            if (((RDFSClass) iter.next()).getURI().equals(range.getURI())) {
                existingRange = true;
                break;
            }
        }

        if (!existingRange) {
            property.getRDFSRange().add(range);
        }
    }

    /**
     * create RDFSComments for RDFSResource
     * 
     * @param onto
     * @param ec
     * @param rs
     */
    private void createComments(Ontology onto, ENamedElement ec, RDFSResource rs) {

        if (onto == null || ec == null || rs == null)
            return;

        Iterator anlist = ec.getEAnnotations().iterator();
        EAnnotation an = null;
        while (anlist.hasNext()) {
            an = (EAnnotation) anlist.next();
            if (an == null)
                continue;
            PlainLiteral comment = null;

            // Get comments from details
            for (int i = 0; i < ((EMap) an.getDetails()).values().toArray().length; i++) {
                if (!((EMap) an.getDetails()).values().toArray()[i].equals("")) {
                    comment = factory.createPlainLiteral();
                    comment.setLexicalForm(((EMap) an.getDetails()).values()
                            .toArray()[i].toString());
                    rs.getRDFSComment().add(comment);
                }
            }
            // If no comments in details, get source
            if (comment == null) {
                comment = factory.createPlainLiteral();
                comment.setLexicalForm(an.getSource());
                rs.getRDFSComment().add(comment);
            }

        }

        // add instanceClassName if EDataType

        if (ec instanceof EDataType) {
            PlainLiteral instanceClassName = factory.createPlainLiteral();
            instanceClassName.setLexicalForm(XSD.ECORE_INSTANCE_CLASS_NAME
                                             + ((EDataType) ec)
                                                     .getInstanceClassName());
            rs.getRDFSComment().add(instanceClassName);
        }
    }

    /**
     * set input file URL. If outputfile's URL is "null", automaticly generate a
     * .rdf file in the same folder as default output file.
     * 
     * @param in
     *            input file's URL
     */
    public void setInputEcoreFile(String in) {

        if (in == null) {
            inputEcoreFile = null;
            return;
        }

        inputEcoreFile = in;

        if (outputRDFFile == null) {
            int p = inputEcoreFile.lastIndexOf(".");
            outputRDFFile = (p != -1 ? inputEcoreFile.substring(0, p + 1)
                                       + "rdf" : inputEcoreFile + ".rdf");
        }

    }

    /**
     * set output file URL
     * 
     * @param out
     *            output file's URL
     */
    public void setOutputRDFFile(String out) {

        if (out == null)
            return;
        outputRDFFile = out;

    }

    /**
     * @return
     */
    public String getInputEcoreFile() {
        return inputEcoreFile;
    }

    /**
     * @return
     */
    public String getOutputRDFFile() {
        return outputRDFFile;
    }

    public static void main(String[] args) {
    	/*
        //new a RDF to Ecore transformer
        ECore2RDFSTransformer rdfs2ecore = new ECore2RDFSTransformer();
        Rose2ECoreTransformer t = new Rose2ECoreTransformer();
        
        //set namespace for ecore model

        try {
            ResourceSet resourceSet = t.convert("./odm.mdl");
            //invoke mapping with inputRDFFile
            rdfs2ecore.convertECore2RDFS(resourceSet,"odm.rdf");
            //rdfs2ecore.converECore2RDF(resourceSet, "testcase/euml2.ecore");
        } catch (Exception e) {
            e.printStackTrace();
        }
        */
    }

}
